/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#ifdef	notdef
#include <gdk/gdkkeysyms.h>
#endif	/*notdef*/
#include "ConsolePanel.h"
#include "node.h"
#include "dialogs.h"
#include <vacmclient_api.h>

#ifndef	__sgi
#ifndef	pid_t
typedef __pid_t pid_t;
#endif	/*pid_t*/
#endif	/*__sgi*/

extern char	**environ;
static void	child_died_event(GtkWidget *term,gpointer data);

#include <zvt/zvtterm.h>

ConsolePanel::ConsolePanel(Node *node,gpointer /*data*/) 
{
	theNode = node;

	theBaseWidget = gtk_table_new(1 /*rows*/,2 /*cols*/,FALSE);
	gtk_table_set_row_spacings((GtkTable *) theBaseWidget,5);
	show();

	GtkWidget *hbox = gtk_hbox_new(FALSE,0);
	gtk_box_set_spacing(GTK_BOX(hbox),2);
	gtk_widget_show(hbox);

	gtk_table_attach(GTK_TABLE(theBaseWidget),hbox,0,1,0,1,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),0,0);

	theContentWidget = zvt_term_new_with_size(80,24);
	gtk_object_set_data(GTK_OBJECT(theContentWidget),"console",(gpointer) this);
	gtk_object_set_data(GTK_OBJECT(theContentWidget),"node",(gpointer) node);
	gtk_widget_set_usize(theContentWidget,-1,100);
	gtk_widget_show(theContentWidget);

	zvt_term_set_blink(ZVT_TERM(theContentWidget), FALSE);
	zvt_term_set_font_name(ZVT_TERM(theContentWidget), "fixed");
	zvt_term_set_bell(ZVT_TERM(theContentWidget), TRUE);
	zvt_term_set_scrollback(ZVT_TERM(theContentWidget), 1);
	zvt_term_set_scroll_on_keystroke(ZVT_TERM(theContentWidget), TRUE);
	zvt_term_set_scroll_on_output(ZVT_TERM(theContentWidget), TRUE);
//	zvt_term_set_background(ZVT_TERM(theContentWidget), NULL, 0, 0);
	zvt_term_set_wordclass(ZVT_TERM(theContentWidget), 
			       (unsigned char *) "-A-Za-z0-9/_:.,?+%=");

	gtk_signal_connect(GTK_OBJECT(theContentWidget),"child_died",
      			   GTK_SIGNAL_FUNC(child_died_event),(gpointer) this);

	GtkWidget *scrollbar = gtk_vscrollbar_new(GTK_ADJUSTMENT(
				ZVT_TERM(theContentWidget)->adjustment));
	GTK_WIDGET_UNSET_FLAGS(scrollbar,GTK_CAN_FOCUS);

	gtk_box_pack_start(GTK_BOX(hbox),theContentWidget,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(hbox),scrollbar,FALSE,TRUE,0);

	gtk_widget_show(scrollbar);

	activateConsole();
}

ConsolePanel::~ConsolePanel() 
{
	theNode->services->doConsoleDisconnect();
}

static void
reallyDoDelete(gpointer data,GtkWidget *)
{
	ConsolePanel	*panel = (ConsolePanel *) data;

	if (panel) delete panel;
}

static void
child_died_event(GtkWidget *term,gpointer data)
{
	ConsolePanel	*panel = (ConsolePanel *) data;
	char	buffer[512];

	sprintf(buffer,"Console for node '%s' died.  \nRemove the panel?  ",
		panel->theNode->name);
	binary_dialog(buffer,GTK_SIGNAL_FUNC(reallyDoDelete),NULL,
		      (gpointer) data);
}

static void
handleNodeConsoleLogging(InputEntry *entry,gpointer clientData)
{
        ConsolePanel	*cons = (ConsolePanel *) clientData;
	char	*cmdbuf[4];
	int	cmds = 0;

        for (InputEntry *tmp = entry; tmp != NULL; tmp = tmp->next) {
		char *connect = strtok(tmp->string,":");
#ifdef	notdef
		if (!strcasecmp(connect,"JOB_ERROR")) {
			cons->theNode->services->doConsoleSteal(
						(gpointer) cons, 
						handleNodeConsoleLogging);
			return;
		}
#endif	/*notdef*/
		if (strcasecmp(connect,"connect")) continue;	// not out stuff
		(void) strtok(NULL,":");	// ??
		char *ip   = strtok(NULL,":");	// ip addr
		char *port = strtok(NULL,":");	// tcp port
		char *otp  = strtok(NULL,":");	// password

		cmdbuf[cmds++] = "/usr/bin/telnet";
		cmdbuf[cmds++] = strdup(ip);
		if (port)
			cmdbuf[cmds++] = strdup(port);
		cmdbuf[cmds++] = NULL;

		/*--------------------------------------*
		 *		Setup Environ		*
		 *--------------------------------------*
		 *					*
		 * A bunch of this is derived from 	*
		 * zterm.c				*
		 *					*
		 *--------------------------------------*/

		/* set up terminal environment */
		char	**p;
		char	**env = environ;
		for (p = env; *p; p++);
		int	i = p - env;
		char	**env_copy = (char **) g_malloc(sizeof(char *) * (i+10));
		for (i = 0, p = env; *p; p++) {
			if (strncmp (*p, "TERM=", 5) == 0)
				env_copy [i++] = "TERM=linux";
			else if (   (!strncmp (*p, "COLUMNS=", 8))
				 || (!strncmp (*p, "LINES=",   6)))
				continue;
			else
				env_copy [i++] = *p;
		}
		env_copy [i++] = "COLORTERM=zterm";
		env_copy [i++] = "COLUMNS=80";
		env_copy [i++] = "LINES=24";
		env_copy [i++] = "TEST";
		env_copy [i] = NULL;

		/*--------------------------------------*
		 *		Let'r Rip!!!		*
		 *--------------------------------------*/
		switch (zvt_term_forkpty(ZVT_TERM(cons->contentWidget()), 
		  			   ZVT_TERM_DO_UTMP_LOG 
					 | ZVT_TERM_DO_WTMP_LOG)) {
		case -1:
			perror("ERROR: unable to fork:");
			exit(1);
			break;
		case 0:
			environ = env_copy;
			execvp(cmdbuf[0], cmdbuf);
			perror("ERROR: Cannot exec command:");
			exit(1);
		default:
			// shovel the password down the pipe to the child
			if (otp) {
				zvt_term_writechild(ZVT_TERM(cons->contentWidget()),
						    otp,strlen(otp));
				zvt_term_writechild(ZVT_TERM(cons->contentWidget()),
						    "\n\n",2);
			}
			break;
		}

		return;
	}
}

#ifdef	notdef
static void
handleNodeConsoleAlerts(InputEntry *entry,gpointer clientData)
{
        ConsolePanel	*cons = (ConsolePanel *) clientData;

        for (InputEntry *tmp = entry; tmp != NULL; tmp = tmp->next) {
		cons->writeStr(tmp->string);
	}
}
#endif	/*notdef*/

void
ConsolePanel::activateConsole()
{
	theNode->services->doConsoleActivate((gpointer) this, 
					     handleNodeConsoleLogging);
#ifdef	notdef
	theNode->services->doConsoleAlerts((gpointer) this, 
					   handleNodeConsoleAlerts);
#endif	/*notdef*/
}

void    
ConsolePanel::writeStr(char *s)
{
	if (s == NULL) return;
	zvt_term_feed(ZVT_TERM(contentWidget()), s, strlen(s));
}
