/*
 * Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     This is the MSC node specific code.
 *
 */

#include "hoover.h"
#include "node.h"
#include "MSCservices.h"
#include "libgessie/Utils.h"

static void
handleNodeConsoleInquiry(InputEntry *entry,gpointer clientData)
{
	Node *node = (Node *) clientData;

	if (entry == NULL) return;
	(void) strtok(entry->string,":");	// skip INQUIRY
	(void) strtok(NULL,":");		// node id
	char *device = strtok(NULL,":");	// device
//	char *login = strtok(NULL,":");		// login
//	char *passwd = strtok(NULL,":");	// password

	node->console = strdup(device ? device : "");
}

void 
MSCservices::handleInquiryMessage(InputEntry *entry)
{
	if (entry == NULL) return;
	(void) strtok(entry->string,":");	// skip NODELIST
	(void) strtok(NULL,":");		// the node id
        char *dev   = strtok(NULL,":");		// the dev
        char *speed = strtok(NULL,":");		// speed

	/*
	 *	NOTE: should check node_id to make sure
	 */
        parent->emp_port = dev ? strdup(dev) : NULL;

	/*
	 *	Do a serial console inquiry.
	 */
	doConsoleInquiry(handleNodeConsoleInquiry);
}

void 
MSCservices::handleInventoryMessage(InputEntry *entry)
{
	if (entry == NULL) return;
	(void) strtok(entry->string,":");        /* NODEINVINFO */
	char *sys = strtok(NULL,":");
	if (!strncasecmp(sys,"permission denied",17)) {
		parent->setState(N_STATE_NOAUTH);
		parent->updateVisual();
		return;
	}
	char *version = strtok(NULL,":");

	if (parent->getState() == N_STATE_NOAUTH) {
		parent->checkState = CHECK_NORMALLY;
		parent->setState(N_STATE_RUNNING);
		parent->updateVisual();
	}

	char    buffer[1024];

	/*
	 *      System name
	 */

//	parent->clearSubtree(); // clear out items from before, if any
	sprintf(buffer,"System = %s %s",sys,version);
	GtkWidget *item = gtk_tree_item_new_with_label(buffer);
	NONSELECT(item);
	parent->addToSubtree(item);
	gtk_widget_show(item);
	parent->collapseSubtree();
}
