/*
 * Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     This handles the tree list of nodes and clusters and such.
 *
 */

#include <stdio.h>
#include <string.h>
#include "MainListPanel.h"
#include "node.h"
#include "connection.h"
#include "libgessie/Gessie.h"
#include "dialogs.h"
#include "node_list.h"
#include "HooverMessages.h"

extern void	doTheMetaClusterAdd(char *metaname,gpointer clientData, 
				    gpointer callData);

extern void	doRSHWatchDialog(gpointer data,GtkWidget *w);

MainListPanel::MainListPanel(PanelContainer * /*p*/)
{
	curSelected = NULL;

	theBaseWidget = gtk_table_new(2 /*rows*/,1 /*cols*/,FALSE);
	gtk_widget_show(theBaseWidget);

	GtkWidget *scrollw = gtk_scrolled_window_new((GtkAdjustment *) NULL,
						     (GtkAdjustment *) NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollw),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	gtk_widget_show(scrollw);
	assert(scrollw != NULL);

	/* Create the root tree */
	theContentWidget = (GtkWidget *) gtk_tree_new();
	gtk_tree_set_selection_mode(GTK_TREE(theContentWidget),GTK_SELECTION_SINGLE);

	gtk_widget_show(theContentWidget);

#ifdef	notdef
	/*----*/
	GtkStyle *style = gtk_style_copy(gtk_widget_get_style(theContentWidget));
	static GdkColor colorGrey  = { 0, 0xeeee, 0xeeee, 0xeeee };
	gdk_color_alloc(gdk_colormap_get_system(), &colorGrey);
	style->base[GTK_STATE_NORMAL] = colorGrey;
	gtk_widget_set_style(theContentWidget,style);
	/*----*/
#endif	/*notdef*/

	/* Add it to the scrolled window */
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollw),
					      theContentWidget);

	gtk_table_attach(GTK_TABLE(theBaseWidget),scrollw,0,1,0,1,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
			 0,0);

	/*--------------------------------------------------------------*
	 *			     BUTTON BOX				*
	 *--------------------------------------------------------------*/

	GtkWidget *hbox = gtk_hbox_new(FALSE /*TRUE*/, 3);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
	gtk_widget_show (hbox);
	gtk_table_attach(GTK_TABLE(theBaseWidget),hbox,0,1,1,2,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
			 GTK_SHRINK, 0,0);

#define BUTTON_PAD      1

#define MK_BUTTON(nm,str,n)                                             \
        nm = gtk_button_new_with_label(str);                            \
        gtk_container_set_border_width (GTK_CONTAINER (nm), 2);         \
        gtk_box_pack_start(GTK_BOX(hbox),nm,FALSE,FALSE,BUTTON_PAD);    \
        gtk_widget_show(nm);                                            \
        gtk_widget_set_sensitive(nm,FALSE);                             \
        gtk_signal_connect_object(GTK_OBJECT(nm), "clicked",            \
                                  GTK_SIGNAL_FUNC(doNodeAction),	\
                                  (GtkObject *) n);                     \
        gtk_object_set_data(GTK_OBJECT(nm),"panel",(void *) this)

        MK_BUTTON(propertiesButton,	"Properties",	PROPERTIES);
        MK_BUTTON(resetButton,		"Reset",	RESET_IT);
        MK_BUTTON(powerdownButton,	"Power Down",	POWER_DOWN);
        MK_BUTTON(powerupButton,	"Power Up",	POWER_UP);
        MK_BUTTON(viewlogButton,	"View Log",	VIEW_LOG);
        MK_BUTTON(serialconsoleButton,	"Console",	SERIAL_CONSOLE);
//      MK_BUTTON(statusButton,		"Status",	STATUS);
}

void
MainListPanel::setButtonSensitivity(gboolean b)
{
	gtk_widget_set_sensitive(propertiesButton,b);
	gtk_widget_set_sensitive(resetButton,b);
	gtk_widget_set_sensitive(powerdownButton,b);
	gtk_widget_set_sensitive(powerupButton,b);
	gtk_widget_set_sensitive(viewlogButton,b);
	gtk_widget_set_sensitive(serialconsoleButton,b);
}

int
MainListPanel::interested(char *msgtype,gpointer data)
{
	Node *node = (Node *) data;
	if (!strcmp(msgtype,HooverMessages.NodeSelected)) {
		setButtonSensitivity(TRUE);
		if (node->type == TYPE_RSH_NODE) {
			/*
			 *	RSH nodes can't do some specific things.
			 */
			gtk_widget_set_sensitive(powerupButton,FALSE);
		}
		if (node->type == TYPE_MSC_NODE) {
			gtk_widget_set_sensitive(viewlogButton,FALSE);
		}
	}
	else if (!strcmp(msgtype,HooverMessages.NodeUnselected)) {
		setButtonSensitivity(FALSE);
	}
	return(0);
}

void
addNodeCallback(gpointer callback_data,guint /*action*/,GtkWidget *)
{
	MainListPanel *panel = (MainListPanel *) callback_data;
	Node    *node;
	GtkWidget       *selected_widget = NULL;
	Node	*tmp;

	if (panel->currentSelection() != NULL) {
		selected_widget = panel->currentSelection();
	}
	else if (g_slist_length(theNodeList) == 1) {
		/*
		 *      We haven't selected anything, but there is only
		 *      one cluster, so use that by default.
		 */
		tmp = (Node *) g_list_nth_data((GList *) theNodeList,0);
		selected_widget = tmp->node_item;
	}
	if (selected_widget == NULL) {
		informational_dialog("You must first select a  \ncluster or sibling node.");
		return;
	}
	node = (Node *) gtk_object_get_data(GTK_OBJECT(selected_widget),"item");        if (node == NULL) {
	//      fprintf(stderr,"addNodeCallback() - node == NULL\n");
		return;
	}
	node->getNodeForCluster();
}

void
doTheClusterAdd(char *hostname,gpointer /*clientData*/,gpointer /*callData*/)
{
	ConnectInfo     *tmp = NULL;
	Node	*node;

	tmp = new ConnectInfo(hostname);
	node = tmp->renderClusterNode();
	theNodeList = g_slist_append(theNodeList, (gpointer) node);
	tmp->position = g_slist_length(theNodeList) - 1;
}

static void
addClusterCallback(gpointer /*callback_data*/,guint /*action*/,GtkWidget *)
{
        get_textentry_dialog("Add New Cluster  ",doTheClusterAdd,
			     (gpointer) NULL, "Hostname  ","Add Cluster");
}

static void
addMetaClusterCallback(gpointer /*callback_data*/,guint /*action*/,GtkWidget *)
{
        get_textentry_dialog("Create Meta Cluster  ",doTheMetaClusterAdd,
			     (gpointer) NULL,"MetaCluster Name  ", 
			     "Add MetaCluster");
}

void
MainListPanel::doMenu(GtkWidget *parent)
{
        GtkWidget *menu_item;

#ifdef	notdef
	/*
	 *	NOTE -	DON'T let the user delete this for now
	 *		This effectively does this by not including
	 *		the 'remove panel' option.
	 */
        Panel::doMenu(parent);          // put standard pieces in menu
#endif	/*notdef*/
        /*--------------------------*/

	PANEL_ADD_MENU_ITEM("Add Node...",addNodeCallback,this);
	PANEL_ADD_MENU_ITEM("Add Cluster...",addClusterCallback,this);
	PANEL_ADD_MENU_ITEM("Add MetaCluster...",addMetaClusterCallback,this);
}

/*
 *
 */

extern void	doAdminDialog(gpointer,GtkWidget *); // Auth.C

void
node_item_button_pressed(GtkWidget *w,GdkEventButton *event)
{
	Node		*node = NULL;
	GtkWidget	*menu_item;
	GtkWidget	*p;
	MainListPanel	*mainList = NULL;

	if (event->button != 3) return;
	node = (Node *) gtk_object_get_data(GTK_OBJECT(w),"item");	
	mainList = (MainListPanel *) gtk_object_get_data(GTK_OBJECT(w),"mainList");	
	if (w == NULL) {
		fprintf(stderr,"got null tree item!\n");
	}
	//	Select correct thing
	gtk_tree_select_child((GtkTree *) theList,w);
	mainList->setCurrentSelection(w);
	p = gtk_menu_new();
	gtk_widget_show(p);

	/*
	 *	Populate the menu
	 */
	
#define MENU_ITEM(par,str,cb,cd)					\
		menu_item = gtk_menu_item_new_with_label(str);		\
		gtk_menu_append(GTK_MENU(par), menu_item);		\
		gtk_signal_connect_object(GTK_OBJECT(menu_item),	\
					  "activate",			\
					  GTK_SIGNAL_FUNC(cb),		\
					  (GtkObject *) cd);		\
		gtk_object_set_data(GTK_OBJECT(menu_item),"main",	\
				    (void *) mainList);			\
		gtk_widget_set_sensitive(menu_item,(cb == NULL)?	\
					 FALSE : TRUE);			\
		gtk_widget_show(menu_item)

#define	MENU_SEP							\
		menu_item = gtk_menu_item_new();			\
		gtk_menu_append(GTK_MENU(p), menu_item);		\
		gtk_widget_show(menu_item)


	if (node->type == TYPE_CLUSTER) {
		MENU_ITEM(p,"Add Node...",	addNodeCallback,mainList);
		MENU_SEP;
	}

	MENU_ITEM(p,"Properties...",	doNodeAction,PROPERTIES);
	MENU_ITEM(p,"View Log...",	doNodeAction,VIEW_LOG);
	if (node->type == TYPE_EMP_NODE) {
		MENU_ITEM(p,"Console...", doNodeAction,SERIAL_CONSOLE);
	}

	MENU_SEP;

	if (node->singularTypeP()) {
		MENU_ITEM(p,"Reset Node",	doNodeAction,RESET_IT);
		MENU_ITEM(p,"Power Down Node",	doNodeAction,POWER_DOWN);
		MENU_ITEM(p,"Power Up Node",	doNodeAction,POWER_UP);
	}
	else	{
		MENU_ITEM(p,"Reset Cluster",	 doNodeAction,RESET_IT);
		MENU_ITEM(p,"Power Down Cluster",doNodeAction,POWER_DOWN);
		MENU_ITEM(p,"Power Up Cluster",	 doNodeAction,POWER_UP);
	}

	MENU_SEP;
	MENU_ITEM(p,"Remove Node",		doNodeAction,REMOVE_IT);

	if (node->type == TYPE_EMP_NODE) {
		MENU_SEP;
		MENU_ITEM(p,"Sensors",		doNodeAction,SENSE_IT);
	}
	else if (node->type == TYPE_RSH_NODE) {
		MENU_SEP;
		MENU_ITEM(p,"Watch Process...",	doRSHWatchDialog,(void *) node);
	}
	else if (node->type == TYPE_CLUSTER) {
		MENU_SEP;
		MENU_ITEM(p,"Authority...",	doAdminDialog,(void *) node->connection);
	}

	/*
	 *	Let'r rip!
	 */
	gtk_menu_popup(GTK_MENU(p),NULL,NULL,NULL,NULL,3,event->time);
	if (GTK_TREE_ITEM(w)->subtree && GTK_TREE_ITEM(w)->expanded) {
		/*
		 *	Now, insure that the item will remain expanded.
		 */
		gtk_signal_emit_stop_by_name(GTK_OBJECT(w),
					     "button_press_event");
		gtk_signal_emit_by_name(GTK_OBJECT(w),"expand");
	}
}
