/*
 * Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     This is the panel that contains the aggregated list
 *		of properties for an entire cluster. A particular node
 *		properties list will come up as a dialog, but for a
 *		cluster, its a spreadsheet format.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include "PropListPanel.h"
#include "node.h"
#include "gtkhacks.h"
#include "input_queue.h"
#include "connection.h"
#include "libgessie/Gessie.h"
#include "dialogs.h"
#include "node_list.h"
#include <assert.h>
#include "ClistHacks.h"
#include "HooverMessages.h"

#define	BOGUS_SORT_COLUMN	999

extern void	createProperties(Node *,gpointer data);
static void	selectRowFromClusterList(gpointer data,gint row,gint column,
                         		 GdkEventButton *event,
					 gpointer user_data);

static struct clist_column_info column_data[] = {
{ "Cluster Name", 100, TRUE, GTK_JUSTIFY_LEFT	},	/* cluster	*/
{ "Node Label",	   80, TRUE, GTK_JUSTIFY_LEFT	},	/* node label	*/
{ "Password",	   60, TRUE, GTK_JUSTIFY_LEFT	},	/* node password*/
{ "Serial Number",100, TRUE, GTK_JUSTIFY_LEFT	},	/* serial number*/
{ "EMP Port",	   60, TRUE, GTK_JUSTIFY_LEFT	},	/* emp port	*/
{ "Node Eth0",	   80, TRUE, GTK_JUSTIFY_LEFT	},	/* eth0		*/
{ "State",	   60, TRUE, GTK_JUSTIFY_LEFT	},	/* state	*/
};

void
PropListPanel::renderData()
{
        char    *recs[7];
	Node	*n = NULL;

	/*------------------------------------------------------*
	 *		Put the data into it.			*
	 *------------------------------------------------------*/

	gtk_clist_freeze(GTK_CLIST(theContentWidget));
	recs[0] = (theNode->connection->hostname) ? theNode->connection->hostname : ((char *) "");
	for (unsigned int i = 0; i < g_slist_length(theNode->childList); i++) {
		n = (Node *) g_list_nth_data((GList *) theNode->childList,i);
		if (n->type == TYPE_ALIAS) {
			n = n->alias;
		}
		recs[1] = n->name;
		recs[2] = (n->password)	    ? n->password : ((char *) "NONE");
		recs[3] = (n->serial_number) ? n->serial_number: ((char *) "");
		recs[4] = (n->emp_port) ? n->emp_port : ((char *) "");
		recs[5] = (n->ip) ? n->ip : ((char *) "");
		recs[6] = n->stateString();
                gtk_clist_prepend(GTK_CLIST(theContentWidget), recs);
		gtk_clist_set_row_data(GTK_CLIST(theContentWidget),0,(gpointer) n);
	}
	gtk_clist_thaw(GTK_CLIST(theContentWidget));
}

PropListPanel::PropListPanel(PanelContainer * /*p*/, Node *node)
{
	int cnt = 7;
	int	i;
	gchar   **clist_titles /*= column_titles*/;
	GtkWidget	*scroller;

	clist_titles = (gchar **) malloc(sizeof(gchar *) * cnt);
	for (i = 0; i < 7; i++) {
		clist_titles[i] = column_data[i].title;
	}

	theNode = node;

//	setParent(p);
	theBaseWidget = gtk_table_new(2 /*rows*/,1 /*cols*/,FALSE);
	gtk_table_set_row_spacings((GtkTable *) theBaseWidget,5);
	show();

	scroller = gtk_scrolled_window_new((GtkAdjustment *) NULL,
					   (GtkAdjustment *) NULL);

	gtk_container_set_border_width(GTK_CONTAINER(scroller), 5);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
				       GTK_POLICY_AUTOMATIC,GTK_POLICY_ALWAYS);
	gtk_widget_show(scroller);
	assert(scroller != NULL);

        gtk_table_attach(GTK_TABLE(theBaseWidget),scroller,0,1,0,1,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND), 
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),0,0);

		/*------------------------------*
		 *		CLIST		*
		 *------------------------------*/

	theContentWidget = gtk_clist_new_with_titles(cnt,clist_titles);
	gtk_clist_set_selection_mode(GTK_CLIST(theContentWidget),
				     GTK_SELECTION_SINGLE);
	gtk_signal_connect_object(GTK_OBJECT(theContentWidget),"select-row",
				  GTK_SIGNAL_FUNC(selectRowFromClusterList),
				  (GtkObject *) this);
	gtk_signal_connect(GTK_OBJECT(theContentWidget), "click-column",
			   GTK_SIGNAL_FUNC(hooverSortOnColumn), 
			   (gpointer) theContentWidget);
	gtk_clist_set_compare_func(GTK_CLIST(theContentWidget),hooverSortCompare);
	gtk_object_set_data(GTK_OBJECT(theContentWidget),"sort_column",
			    (void *) BOGUS_SORT_COLUMN);
        gtk_container_add(GTK_CONTAINER(scroller), theContentWidget);
	gtk_widget_show(theContentWidget);
	gtk_clist_set_shadow_type(GTK_CLIST(theContentWidget),GTK_SHADOW_OUT);
	for (i = 0; i < cnt; i++) {
		gtk_clist_set_column_width(GTK_CLIST(theContentWidget),i,
					   column_data[i].width);
		gtk_clist_set_column_justification(GTK_CLIST(theContentWidget),
					   i,column_data[i].justification);
	}

	renderData();
	gtk_widget_show(theBaseWidget);
}

int
PropListPanel::interested(char *msgtype,gpointer data)
{
	if (!strcmp(HooverMessages.NodelistChanged,msgtype)) {
		Node  *node = (Node *) data;
		if (node != theNode) return(0);
		gtk_clist_clear(GTK_CLIST(theContentWidget));
		renderData();
		return(0);
	}
	return(0);
}

void
PropListPanel::doMenu(GtkWidget *parent)
{
        Panel::doMenu(parent);          // put standard pieces in menu
        /*--------------------------*/
}

static void
selectRowFromClusterList(gpointer data,gint row,gint /*column*/,
                         GdkEventButton * /*event*/,gpointer /*user_data*/)
{
	PropListPanel	*plp = (PropListPanel *) data;
	Node	*node = (Node *) gtk_clist_get_row_data(GTK_CLIST(plp->contentWidget()),row);
	if (node == NULL) {
		// put up error message here
		return;
	}
	createProperties(node,NULL);
}

