/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include "node.h"
#include "alert.xpm"
#include "question.xpm"

/*
 *	Something to squirrel away data for binary
 */

typedef void (*action) (gpointer data,GtkWidget *w);

typedef	struct	dialogSquirrel_t {
	GtkSignalFunc	yes;
	GtkSignalFunc	no;
	gpointer	clientData;
	GtkWidget	*dialog;
} DialogSquirrel;

extern GtkWidget	*theRootWindow;

GtkWidget *
getIcon(gchar **data)
{
	GtkStyle *style;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GtkWidget *tmp;

	style = gtk_widget_get_style(theRootWindow);
	pixmap = gdk_pixmap_create_from_xpm_d(theRootWindow->window,  &mask,
					      &style->bg[GTK_STATE_NORMAL],
					      data);
	tmp = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(tmp);
	return(tmp);
}

/* Function to open a dialog box displaying the message provided. */

#ifdef	notdef
#include <libgnomeui/gnome-messagebox.h>
#include <libgnomeui/gnome-stock.h>
#endif	/*notdef*/

GtkWidget * 
informational_dialog(gchar *message) 
{
#ifdef	notdef
	GtkWidget *mbox;

	mbox = gnome_message_box_new(message,GNOME_MESSAGE_BOX_WARNING,
				     GNOME_STOCK_BUTTON_OK,NULL);
	gtk_widget_show(mbox);
#else	/*notdef*/

	/* Create the widgets */

	GtkWidget *dialog = gtk_dialog_new();
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox),5);

	/*
	 *	Top portion
	 */
        GtkWidget *hbox = gtk_hbox_new(FALSE /*TRUE*/, 3);
	gtk_box_set_spacing(GTK_BOX(hbox),10);
        gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
        gtk_widget_show (hbox);

	GtkWidget *icon = getIcon((gchar **) alert_xpm);
	gtk_container_add(GTK_CONTAINER(hbox),icon);

	GtkWidget *label = gtk_label_new (message);
	gtk_container_add(GTK_CONTAINER(hbox),label);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),hbox);

	/*
	 *	Button portion
	 */
	GtkWidget *okay_button = gtk_button_new_with_label("Okay");

	/* Ensure that the dialog box is destroyed when the user clicks ok. */

	gtk_signal_connect_object(GTK_OBJECT (okay_button), "clicked",
		       		  GTK_SIGNAL_FUNC (gtk_widget_destroy), 
				  (GtkObject *) dialog);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  okay_button);

	/* Add the label, and show everything we've added to the dialog. */
	gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_MOUSE);
	gtk_widget_show_all (dialog);
        gtk_object_set_data(GTK_OBJECT(dialog),"label",(void *) label);
	while (gtk_events_pending()) gtk_main_iteration();	// spin dry!
	return(dialog);
#endif	/*notdef*/
}

void
informational_dialog_update(GtkWidget *dialog,gchar *message) 
{
	GtkWidget *label = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(dialog),
							     "label");
	gtk_label_set_text((GtkLabel *) label,message);	
        while (gtk_events_pending()) gtk_main_iteration();      // spin dry!
}

static void
gottaYes(GtkWidget *w,gpointer data)
{
	DialogSquirrel	*tmp = (DialogSquirrel *) data;
	action	func;
	gpointer	clientData;

	gtk_grab_remove(GTK_WIDGET(w));
	if (tmp == NULL) {	// nothing to do
		free(tmp);	/* free up stuff and leave */
		return;
	}
	func = (action) tmp->yes;
	clientData = tmp->clientData;
	gtk_widget_destroy(tmp->dialog);
	free(tmp);		// clean up 
	if (func != NULL) 
		func(clientData,w);
}

static void
gottaNo(GtkWidget *w,gpointer data)
{
	DialogSquirrel	*tmp = (DialogSquirrel *) data;
	action	func;
	gpointer	clientData;

	gtk_grab_remove(GTK_WIDGET(w));
	if (tmp == NULL) {	// nothing to do
		free(tmp);	/* free up stuff and leave */
		return;
	}
	func = (action) tmp->no;
	clientData = tmp->clientData;
	gtk_widget_destroy(tmp->dialog);
	free(tmp);		// clean up 
	if (func != NULL) 
		func(clientData,w);
}

void 
binary_dialog(gchar *msg,GtkSignalFunc yes_cb,GtkSignalFunc no_cb, 
	      gpointer clientData) 
{
	DialogSquirrel	*tmp;

	/* Create the widgets */

	GtkWidget *dialog = gtk_dialog_new();
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox),5);

	/*
	 *	Stash away some stuff for later
	 */

	tmp = (DialogSquirrel *) malloc(sizeof(DialogSquirrel));
	tmp->yes = yes_cb;
	tmp->no  = no_cb;
	tmp->clientData  = clientData;
	tmp->dialog = dialog;

	/*
	 *	Top portion
	 */
        GtkWidget *hbox = gtk_hbox_new(FALSE /*TRUE*/, 3);
	gtk_box_set_spacing(GTK_BOX(hbox),10);
        gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
        gtk_widget_show (hbox);

	GtkWidget *icon = getIcon((gchar **) question_xpm);
	gtk_container_add(GTK_CONTAINER(hbox),icon);

	GtkWidget *label = gtk_label_new(msg);
	gtk_container_add(GTK_CONTAINER(hbox),label);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),hbox);

	/*
	 *	Button portion
	 */
	GtkWidget *yes_button = gtk_button_new_with_label("Yes");
	gtk_signal_connect(GTK_OBJECT(yes_button), "clicked",
		       	   GTK_SIGNAL_FUNC (gottaYes), (gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  yes_button);
	GTK_WIDGET_SET_FLAGS(yes_button,GTK_CAN_DEFAULT);
	gtk_widget_grab_default(yes_button);

	GtkWidget *no_button = gtk_button_new_with_label("No");
	gtk_signal_connect(GTK_OBJECT(no_button), "clicked",
		       	   GTK_SIGNAL_FUNC (gottaNo), (gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  no_button);

	/* Add the label, and show everything we've added to the dialog. */
	gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_MOUSE);
	gtk_widget_show_all(dialog);
}

/*----------------------------------------------------------------------*
 *			    GET TEXT DIALOG				*
 *----------------------------------------------------------------------*/

typedef struct {
        void (*callback)(char *hostname,gpointer clientData,gpointer callData);
        gpointer        clientData;
        gpointer        callData;
        GtkWidget       *dialog;
        GtkWidget       *hostname;
} GetTextDialogSquirrel;

static void
gottaTextCancel(GtkWidget *,gpointer data)
{
	GtkWidget	*dialog;
	GetTextDialogSquirrel *tmp = (GetTextDialogSquirrel *) data;
	dialog = tmp->dialog;
	free(tmp);
        gtk_widget_destroy(dialog);
}

static void
gottaTextAdd(GtkWidget *,gpointer data)
{
	gchar	*txt;
	GtkWidget	*dialog;
	GetTextDialogSquirrel *tmp = (GetTextDialogSquirrel *) data;
	char	*shostname;

	dialog = tmp->dialog;
	if (tmp->callback) {
		txt = (tmp->hostname != NULL) ? gtk_entry_get_text((GtkEntry *) tmp->hostname) : NULL;
		shostname = (txt != NULL && strlen(txt)) ? strdup((char *) txt) 
							 : strdup("");
		tmp->callback(shostname,tmp->clientData,tmp->callData);
	}

	/*
	 *	clean up and get out
	 */
	free(tmp);
        gtk_widget_destroy(dialog);
}

void 
get_textentry_dialog(gchar *msg,
		     void (*callback)(char *hostname,gpointer clientData,
				      gpointer callData),
		     gpointer clientData,gchar *field_msg,gchar *ok_string,
		     gchar *default_text) 
{
	GetTextDialogSquirrel	*tmp;
	int	vPos = 0;

	/* Create the widgets */

	GtkWidget *dialog = gtk_dialog_new();
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox),5);

	/*------------------------------------------------------*
	 *			Top portion			*
	 *------------------------------------------------------*/

        GtkWidget *hbox = gtk_hbox_new(FALSE /*TRUE*/, 3);
	gtk_box_set_spacing(GTK_BOX(hbox),10);
        gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
        gtk_widget_show (hbox);

	GtkWidget *icon = getIcon((gchar **) question_xpm);
	gtk_container_add(GTK_CONTAINER(hbox),icon);

	GtkWidget *label = gtk_label_new(msg);
	gtk_container_add(GTK_CONTAINER(hbox),label);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),hbox);

	/*------------------------------------------------------*
	 *			Data portion			*
	 *------------------------------------------------------*/

        GtkWidget *table = gtk_table_new(1 /*rows*/,2 /*cols*/,FALSE);
	gtk_table_set_row_spacings((GtkTable *) table,5);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);


	label = gtk_label_new(field_msg);
	gtk_label_set_justify((GtkLabel *) label,GTK_JUSTIFY_LEFT);
	gtk_table_attach(GTK_TABLE(table),label,0,1,vPos,
			 vPos+1,GTK_SHRINK, GTK_SHRINK, 0,0);

        GtkWidget *hostname = gtk_entry_new_with_max_length(128);
	if (default_text != NULL) {
		gtk_entry_set_text(GTK_ENTRY(hostname), default_text);
	}
        gtk_entry_set_editable(GTK_ENTRY(hostname), TRUE);
        gtk_table_attach_defaults(GTK_TABLE(table),hostname,1,2,vPos,vPos+1);

	gtk_signal_connect_object(GTK_OBJECT(hostname), "activate",
			    	  GTK_SIGNAL_FUNC(gtk_window_activate_default),
			    	  GTK_OBJECT(dialog));

        gtk_widget_show(hostname);
        gtk_widget_show(label);
	vPos++;

	/*------------------------------------------------------*
	 *			Button portion			*
	 *------------------------------------------------------*/

	/*
	 *	Stash away some stuff for later
	 */
	tmp = (GetTextDialogSquirrel *) malloc(sizeof(GetTextDialogSquirrel));
	tmp->dialog     = dialog;
	tmp->callback   = callback;
	tmp->clientData = clientData;
	tmp->callData	= NULL;
	tmp->hostname	= hostname;

	GtkWidget *add_button = gtk_button_new_with_label(ok_string);
	gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
		       	   GTK_SIGNAL_FUNC (gottaTextAdd), (gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  add_button);
	GTK_WIDGET_SET_FLAGS(add_button,GTK_CAN_DEFAULT);
	gtk_widget_grab_default(add_button);

	GtkWidget *cancel_button = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
		       	   GTK_SIGNAL_FUNC (gottaTextCancel), (gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  cancel_button);

	/* Add the label, and show everything we've added to the dialog. */
	gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_MOUSE);
	gtk_widget_show_all(dialog);
}

/*----------------------------------------------------------------------*
 *			    GET PASSWORD DIALOG				*
 *----------------------------------------------------------------------*/

typedef struct {
        void (*callback)(char *id,char *pw,char *pw2,gpointer clientData,gpointer callData);
        gpointer        clientData;
        gpointer        callData;
        GtkWidget       *dialog;
        GtkWidget       *id;
        GtkWidget       *pw;
        GtkWidget       *pw2;
} GetPasswordDialogSquirrel;

static void
gottaPasswordCancel(GtkWidget *,gpointer data)
{
	GtkWidget	*dialog;
	GetPasswordDialogSquirrel *tmp = (GetPasswordDialogSquirrel *) data;
	dialog = tmp->dialog;
	free(tmp);
        gtk_widget_destroy(dialog);
}

static void
gottaPasswordAdd(GtkWidget *,gpointer data)
{
	gchar	*id_txt;
	gchar	*pw_txt;
	gchar	*pw2_txt;
	GtkWidget	*dialog;
	GetPasswordDialogSquirrel *tmp = (GetPasswordDialogSquirrel *) data;

	dialog = tmp->dialog;
	if (tmp->callback) {
		char	*sid;
		char	*spw;
		char	*spw2;
		id_txt = (tmp->id != NULL) ? gtk_entry_get_text((GtkEntry *) tmp->id) : NULL;
		sid = (id_txt != NULL && strlen(id_txt)) 
						? strdup((char *) id_txt) 
						: strdup("");
		pw_txt = (tmp->pw != NULL) ? gtk_entry_get_text((GtkEntry *) tmp->pw) : NULL;
		spw = (pw_txt != NULL && strlen(pw_txt)) 
						? strdup((char *) pw_txt) 
						: strdup("");
		pw2_txt = (tmp->pw2 != NULL) ? gtk_entry_get_text((GtkEntry *) tmp->pw2) : NULL;
		spw2 = (pw2_txt != NULL && strlen(pw2_txt)) 
						  ? strdup((char *) pw2_txt) 
						  : strdup("");

		tmp->callback(sid,spw,spw2,tmp->clientData,tmp->callData);
	}

	/*
	 *	clean up and get out
	 */
	free(tmp);
        gtk_widget_destroy(dialog);
}

static void
gottaPasswordDefault(GtkWidget *,gpointer data)
{
	GtkWidget	*dialog;
	GetPasswordDialogSquirrel *tmp = (GetPasswordDialogSquirrel *) data;

	dialog = tmp->dialog;
	if (tmp->callback) {
		tmp->callback(NULL,NULL,NULL,tmp->clientData,tmp->callData);
	}

	/*
	 *	clean up and get out
	 */
	free(tmp);
        gtk_widget_destroy(dialog);
}

void 
get_passwordentry_dialog(gchar *msg,
		         void (*callback)(char *id,char *pw,char *pw2,
					  gpointer clientData,
				          gpointer callData),
			 char *default_userid,
			 char *pw2_msg,
			 char *group_msg,
		         gpointer clientData)
{
	GetPasswordDialogSquirrel	*tmp;
	int	vPos = 0;

	/* Create the widgets */

	GtkWidget *dialog = gtk_dialog_new();
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox),5);

	/*------------------------------------------------------*
	 *			Top portion			*
	 *------------------------------------------------------*/

        GtkWidget *hbox = gtk_hbox_new(FALSE /*TRUE*/, 3);
	gtk_box_set_spacing(GTK_BOX(hbox),10);
        gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
        gtk_widget_show (hbox);

	GtkWidget *icon = getIcon((gchar **) question_xpm);
	gtk_container_add(GTK_CONTAINER(hbox),icon);

	GtkWidget *label = gtk_label_new(msg);
	gtk_container_add(GTK_CONTAINER(hbox),label);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),hbox);

	/*------------------------------------------------------*
	 *			Data portion			*
	 *------------------------------------------------------*/

        GtkWidget *table = gtk_table_new(3 /*rows*/,2 /*cols*/,FALSE);
	gtk_table_set_row_spacings((GtkTable *) table,5);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);


	label = gtk_label_new("Userid");
	gtk_label_set_justify((GtkLabel *) label,GTK_JUSTIFY_LEFT);
	gtk_table_attach(GTK_TABLE(table),label,0,1,vPos,
			 vPos+1,GTK_SHRINK, GTK_SHRINK, 0,0);

        GtkWidget *userid = gtk_entry_new_with_max_length(128);
	if (default_userid != NULL) {
		gtk_entry_set_text(GTK_ENTRY(userid), default_userid);
	}
        gtk_entry_set_editable(GTK_ENTRY(userid), TRUE);
        gtk_table_attach_defaults(GTK_TABLE(table),userid,1,2,vPos,vPos+1);

        gtk_widget_show(userid);
        gtk_widget_show(label);
	vPos++;

	label = gtk_label_new("Password");
	gtk_label_set_justify((GtkLabel *) label,GTK_JUSTIFY_LEFT);
	gtk_table_attach(GTK_TABLE(table),label,0,1,vPos,
			 vPos+1,GTK_SHRINK, GTK_SHRINK, 0,0);

        GtkWidget *password = gtk_entry_new_with_max_length(128);
	gtk_entry_set_visibility(GTK_ENTRY(password), FALSE);
        gtk_entry_set_editable(GTK_ENTRY(password), TRUE);
        gtk_table_attach_defaults(GTK_TABLE(table),password,1,2,vPos,vPos+1);
	
        gtk_widget_show(password);
        gtk_widget_show(label);
	vPos++;

	GtkWidget *password2 = NULL;
	if (pw2_msg != NULL) {
		label = gtk_label_new(pw2_msg);
		gtk_label_set_justify((GtkLabel *) label,GTK_JUSTIFY_LEFT);
		gtk_table_attach(GTK_TABLE(table),label,0,1,vPos,
			 	 vPos+1,GTK_SHRINK, GTK_SHRINK, 0,0);

        	password2 = gtk_entry_new_with_max_length(128);
		gtk_entry_set_visibility(GTK_ENTRY(password2), FALSE);
		gtk_entry_set_editable(GTK_ENTRY(password2), TRUE);
        	gtk_table_attach_defaults(GTK_TABLE(table),password2,1,2,vPos,vPos+1);
	
		gtk_widget_show(password2);
		gtk_widget_show(label);
		vPos++;
	}

	/*------------------------------------------------------*
	 *			Button portion			*
	 *------------------------------------------------------*/

	/*
	 *	Stash away some stuff for later
	 */
	tmp = (GetPasswordDialogSquirrel *) malloc(sizeof(GetPasswordDialogSquirrel));
	tmp->dialog     = dialog;
	tmp->callback   = callback;
	tmp->clientData = clientData;
	tmp->callData	= NULL;
	tmp->id		= userid;
	tmp->pw		= password;
	tmp->pw2	= password2;

	/*
	 *
	 */
	GtkWidget *add_button = gtk_button_new_with_label("OK");
	gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
		       	   GTK_SIGNAL_FUNC (gottaPasswordAdd), (gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  add_button);
	GTK_WIDGET_SET_FLAGS(add_button,GTK_CAN_DEFAULT);
	gtk_widget_grab_default(add_button);

	/*
	 *
	 */
	if (group_msg != NULL) {
		GtkWidget *def_button = gtk_button_new_with_label("Use Defaults");
		gtk_signal_connect(GTK_OBJECT(def_button), "clicked",
				   GTK_SIGNAL_FUNC(gottaPasswordDefault),(gpointer) tmp);
		gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
				  def_button);
	}

	/*
	 *
	 */
	GtkWidget *cancel_button = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
		       	   GTK_SIGNAL_FUNC (gottaPasswordCancel), (gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  cancel_button);

	/* Add the label, and show everything we've added to the dialog. */
	gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_MOUSE);
	gtk_widget_show_all(dialog);
}
