/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     The Panel is a key portion of the infrastructure that
 *              libgessie provides. It encapsulates each of the various
 *		bits of discrete visual interface and functionality.
 *
 */

#include "Gessie.h"
#include "Broker.h"
#include "GeneralMessages.h"
#include <string.h>

extern void     addPanelToBrokerList(Panel *);

Panel::Panel() 
{
	self_destruct = FALSE;
	parent = NULL;
	listeners = NULL;
}

char *
Panel::getLabel()
{
	GtkWidget	*tab;
	GtkWidget	*label;

	PanelContainer *pc = (PanelContainer *) gsSearchPanelsUpward(this,"PanelContainer");

	tab = gtk_notebook_get_tab_label((GtkNotebook *) pc->getNotebook(),
					 theBaseWidget);
	label = (GtkWidget *) gtk_object_get_data((GtkObject *) tab,"label");
	if (label != NULL) {
		char	*lab;
		gtk_label_get((GtkLabel *) label,&lab);
		return(strdup(lab));
	}
	return(NULL);
}

void
Panel::setLabel(char *str) 
{
	GtkWidget	*tab;
	GtkWidget	*label;

	PanelContainer *pc = (PanelContainer *) gsSearchPanelsUpward(this,"PanelContainer");

	tab = gtk_notebook_get_tab_label((GtkNotebook *) pc->getNotebook(),
					 theBaseWidget);
	label = (GtkWidget *) gtk_object_get_data((GtkObject *) tab,"label");

	if (label != NULL) {
		gtk_label_set_text((GtkLabel *) label,str);
	}
}

static void
doRemovePanel(gpointer client_data,GtkWidget *)
{
	Panel	*panel = (Panel *) client_data;

	gsBroadcastBrokerMsg(panel,GeneralMessages.PanelDying,client_data);
	delete panel;
}

void
Panel::doMenu(GtkWidget *parent)
{
	GtkWidget *menu_item;

#define	MENU_ITEM(par,str,cb)						\
			menu_item = gtk_menu_item_new_with_label(str);	\
			gtk_menu_append(GTK_MENU(par), menu_item);	\
			gtk_signal_connect_object(GTK_OBJECT(menu_item),\
						  "activate",		\
                                  		  GTK_SIGNAL_FUNC(cb),	\
                                  		  (GtkObject *) this);	\
			gtk_widget_set_sensitive(menu_item,(cb == NULL)?\
						 FALSE : TRUE);		\
			gtk_widget_show(menu_item)

	MENU_ITEM(parent,"Remove Panel",doRemovePanel);
#ifdef	notdef
	MENU_ITEM(parent,"Print",NULL);
	MENU_ITEM(parent,"Save",NULL);
	MENU_ITEM(parent,"Save As...",NULL);
	MENU_ITEM(parent,"Reload Save File",NULL);
	MENU_ITEM(parent,"Container Actions",NULL);
	GtkWidget *container_actions = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item),container_actions);
	MENU_ITEM(container_actions,"Split Vertically",NULL);
	MENU_ITEM(container_actions,"Split Horizontally",NULL);
	MENU_ITEM(container_actions,"Remove Container",NULL);
	MENU_ITEM(container_actions,"Hide Tabs",NULL);
#endif	/*notdef*/

	menu_item = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(parent), menu_item);
	gtk_widget_show(menu_item);

}

void
Panel::remove()
{
        PanelContainer *pc = (PanelContainer *) gsSearchPanelsUpward(this,	
							"PanelContainer");
	if (pc == NULL) return;
        /*
         *      Bring it forward
         */
        gint pageNum = gtk_notebook_page_num((GtkNotebook *) pc->getNotebook(),
                                             baseWidget());
	if (pageNum >= 0) {
		gtk_notebook_remove_page((GtkNotebook *) pc->getNotebook(),
					 pageNum);
		gtk_widget_draw(GTK_WIDGET(pc->getNotebook()), NULL);
	}
	else	{
		// error condition, couldn't find page
	}
	removePanelFromBrokerList((void *) this);
}

void
Panel::raise()
{
        PanelContainer *pc = (PanelContainer *) gsSearchPanelsUpward(this,	
							"PanelContainer");
	if (pc == NULL) return;
        /*
         *      Bring it forward
         */
        gint pageNum = gtk_notebook_page_num((GtkNotebook *) pc->getNotebook(),
                                             baseWidget());
        gtk_notebook_set_page((GtkNotebook *) pc->getNotebook(),pageNum);
}
