/*
 * Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

#define	HOOVER

#include <stdio.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <termios.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <gtk/gtk.h>
#include "hoover.h"
#include "node.h"
#include "dialogs.h"
#include "vacmclient_api.h"
#include "libgessie/Broker.h"
#include "HooverControl.h"
extern HooverControl *theRootHoover;

//#define	DEFAULT_SCAN_MASK	"192.168.0.0/24"

static int	do_nexxus_scan(GtkWidget *w,gchar * text);
static int	init_network_nexxus_scan(GtkWidget *,unsigned int UI_addr, 
					unsigned int UI_mask);

class ScanState {
public:
	ScanState()	{};
	gint	timertag;
	int	I_sock;
	unsigned int	UI_currentip;
	unsigned int	UI_lastip;
	unsigned int	UI_cnt;
	unsigned int	UI_maxcnt;
	unsigned int	UI_numnexxuss;
	GtkWidget	*progressbar;
};

static int	nexxus_scan_timer_cb(gpointer data);

#define	SPIN_DRY	while (gtk_events_pending()) gtk_main_iteration()


void
doNexxusScan(char *text, gpointer /*clientData*/,gpointer /*callData*/)
{
	do_nexxus_scan(NULL/*widget*/,text);
}

static int
do_nexxus_scan(GtkWidget *w,gchar * text)
{
	unsigned int UI_mask;
	char SZ_copy[64];
	char *bword;
	char SZ_networkaddr[32];

	strcpy(SZ_copy, text);
	if (!(bword = strtok(SZ_copy, "/"))) {
		fprintf(stderr,"do_nexxus_scan(): Specified network does not have an IPV4 netmask\n");
		return (-1);
	}
	strcpy(SZ_networkaddr, bword);
	if (!(bword = strtok(NULL, "/"))) {
		fprintf(stderr,"do_nexxus_scan(): Specified network does not have an IPV4 netmask\n");
		return (-1);
	}
	UI_mask = atoi(bword);
//	fprintf(stderr,"do_nexxus_scan(): Scanning network %s (mask %d)\n",
//		SZ_networkaddr,UI_mask);
	init_network_nexxus_scan(w,ntohl(inet_addr(SZ_networkaddr)), UI_mask);
	return (0);
}


static int
init_network_nexxus_scan(GtkWidget *,unsigned int UI_addr, unsigned int UI_mask)
{
	int	I_optval = 1;

	ScanState *state = new ScanState;

	/* Make UDP socket for transmission */
	if ((state->I_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		printf("init_network_nexxus_scan(): Unable to create socket (%m)\n");
		return (-1);
	}
	if (setsockopt(state->I_sock,SOL_SOCKET,SO_REUSEADDR,&I_optval,
			sizeof(I_optval)) < 0) {
		fprintf(stderr,"init_network_nexxus_scan(): setsockopt failure (%m)\n");
		return (-1);
	}

	unsigned int UI_tmpmask = htonl(0xffffffff << UI_mask);
	state->UI_currentip = UI_addr;
	state->UI_lastip = ((state->UI_currentip | UI_tmpmask) - 1);
	state->UI_cnt 	 = 0;
	state->UI_maxcnt = (state->UI_lastip - state->UI_currentip);

	state->progressbar = hooverGetProgressBar();
	gtk_progress_set_format_string(GTK_PROGRESS(state->progressbar),"%p%%");
	gtk_progress_set_show_text(GTK_PROGRESS(state->progressbar), TRUE);

	state->timertag = gtk_timeout_add(100, nexxus_scan_timer_cb, 
					  (void *) state);
	return (0);
}

static gboolean
legitConnect(ScanState * /*state*/,struct sockaddr_in *sockaddr)
{
	int	I_tmpsock;
	int	I_optval = 1;

	if ((I_tmpsock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		return(FALSE);
	}
	SPIN_DRY;	// update the GUI
	if (setsockopt(I_tmpsock, SOL_SOCKET, SO_REUSEADDR, &I_optval,
		       sizeof(I_optval)) <0) {
		return(FALSE);
    	}

	SPIN_DRY;	// update the GUI
	if (connect(I_tmpsock,(struct sockaddr *) sockaddr,sizeof(struct sockaddr)) <0) {
		close(I_tmpsock);
		return(FALSE);
	}


	return(TRUE);
}

static gboolean
alreadyConnected(char *host)
{
	/*--------------------------------------------------------------*
	 *		      See if it already exists			*
	 *--------------------------------------------------------------*/

	void *handle;
	int ret = api_nexxus_return_handle(&handle, host);
	if (ret != API_RETURN_OK) {	// already registered, so don't rep
		return(FALSE);
	}
	if (handleToConnectInfo(handle)) return(TRUE);
	return(FALSE);
}

static int
nexxus_scan_timer_cb(gpointer data)
{
	struct sockaddr_in sock;
	char	buf[128];
	ScanState	*state = (ScanState *) data;

	if (((state->UI_currentip & 0xff) == 0x0) ||
	    ((state->UI_currentip & 0xff) == 0xff)) {
		state->UI_currentip++;
		state->UI_cnt++;
	}
	/* 
	 *	Send ping packet to address
	 */
	memset(&sock, 0, sizeof(sock));
	sock.sin_family = AF_INET;
	sock.sin_port   = htons(6442);
	sock.sin_addr.s_addr = htonl(state->UI_currentip);

	sprintf(buf,"scanning addr %s",inet_ntoa(sock.sin_addr));
	hooverSetStatus(buf);

//	fprintf(stderr,"gtk_progress_configure(%d,%d)\n",state->UI_cnt,
//		state->UI_maxcnt);
	gtk_progress_configure(GTK_PROGRESS(state->progressbar),state->UI_cnt,1,
	 		       state->UI_maxcnt);
	SPIN_DRY;	// update the GUI

	// shortcut discovery process
	if (   legitConnect(state,&sock) 
	    && !alreadyConnected(inet_ntoa(sock.sin_addr))) {
		SPIN_DRY;
		/*
		 *	Check to see if we are already there
		 */
		api_nexxus_suppress_error_messages = 1;
		ConnectInfo *tmp = new ConnectInfo(inet_ntoa(sock.sin_addr));
		if (tmp->connect(1,NULL,NULL,TRUE)) {
			Node *node = tmp->renderClusterNode();
			theNodeList = g_slist_append(theNodeList,(gpointer) node);
			tmp->position = g_slist_length(theNodeList) - 1;
		}
		else	{
			delete tmp;
		}
		api_nexxus_suppress_error_messages = 0;
	}

	state->UI_currentip++;
	state->UI_cnt++;

	if (state->UI_currentip <= state->UI_lastip) {
		return(TRUE);
	}
	/*
	*	Done with stuff, clean up.
	*/
	hooverResetProgressBar();
	hooverSetStatus("");

	gtk_timeout_remove(state->timertag);
	free(state);
	return (FALSE);
}

