/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     This is most of the node specific code.
 *
 * Notes:       I am in the process of further abstracting the concept
 *		of 'node' and will have subclasses such as "EMP_NODE"
 *		and eventually other types.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <gtk/gtk.h>
#include "node.h"
#include "question.xpm"
#include "gtkhacks.h"
#include "MainListPanel.h"
#include "hoover.h"
#include "dialogs.h"
#include "libgessie/Broker.h"
#include "EMPservices.h"
#include "RSHservices.h"
#include "MSCservices.h"

extern MainListPanel *mainList;
extern void     addNodeCallback(gpointer,guint,GtkWidget *);
static void	handleNodeConfiguredMessage(InputEntry *entry,gpointer);

#define SIGCON(a,b)     gtk_signal_connect(GTK_OBJECT(a), b,            \
				   GTK_SIGNAL_FUNC(hooverNodeSelection), b)

static GdkColor colorYellow = { 0, 0xffff, 0xffff, 0x0000 };
static GdkColor colorBlack  = { 0, 0x0000, 0x0000, 0x0000 };
static GdkColor colorLiteGrey  = { 0, 0xeeee, 0xeeee, 0xeeee };
static gboolean	colorsAlloced = FALSE;

static void
doColorAlloc()
{
	gdk_color_alloc(gdk_colormap_get_system(), &colorYellow);
	gdk_color_alloc(gdk_colormap_get_system(), &colorBlack);
	gdk_color_alloc(gdk_colormap_get_system(), &colorLiteGrey);
	colorsAlloced = TRUE;
}

GSList	*
g_slist_remove_all(GSList *list,void *target)
{
	int	ocnt;
	int	cnt;

	if (list == NULL) return(NULL);

	cnt = g_slist_length(list);
	do      {
		ocnt = cnt;
		list = g_slist_remove(list,target);
		cnt  = g_slist_length(theNodeList);
		} while (ocnt != cnt);
	
	return(list);
}

NodeState
stateStringToEnum(char *str)
{
	if (str == NULL)			return(N_STATE_UNKNOWN);
	if (!strcmp(str,"NEGOTIATING"))		return(N_STATE_NEGOTIATING);
	if (!strcmp(str,"WAITING"))		return(N_STATE_WAITING);
	if (!strcmp(str,"DEAD"))		return(N_STATE_DEAD);
	if (!strcmp(str,"RUNNING"))		return(N_STATE_RUNNING);
	if (!strcmp(str,"INITIALIZING"))	return(N_STATE_INITIALIZING);
	if (!strcmp(str,"BOOTING"))		return(N_STATE_BOOTING);
	if (!strcmp(str,"NOAUTH"))		return(N_STATE_NOAUTH);
	return(N_STATE_UNKNOWN);
}

char *
Node::stateString()
{
	switch (state) {
	case N_STATE_CONNECTING:	return("CONNECTING");		
	case N_STATE_NEGOTIATING:	return("NEGOTIATING");		
	case N_STATE_WAITING:		return("WAITING");		
	case N_STATE_DEAD:		return("DEAD");		
	case N_STATE_RUNNING:		return("RUNNING");		
	case N_STATE_INITIALIZING:	return("INITIALIZING");
	case N_STATE_BOOTING:		return("BOOTING");
	case N_STATE_NOAUTH:		return("NO AUTHORIZATION");
	default:			return("UNKNOWN");
	}
}

void
Node::print()
{
	fprintf(stderr,"name->%s\n",name);
	fprintf(stderr,"item->0x%08x\n",(unsigned int) node_item);
	fprintf(stderr,"sernum->%s\n",serial_number);
	fprintf(stderr,"empptr->%s\n",emp_port);
	fprintf(stderr,"state->%d\n",state);
	fprintf(stderr,"type->%d\n",type);
}

extern void checkClusterPropertyStates(Node *changedNode);

void
Node::updateVisual()
{
	GtkWidget	*label;
	char	buffer[512];

	assert(node_item != NULL);
	label = (GtkWidget *) gtk_tree_item_get_label(node_item);
	if (label == NULL) return;	// punt! can't find it
	strcpy(buffer,name);
	if (globalVerboseMode >= 2) {
		strcat(buffer,"-");
		strcat(buffer,nodeTypeName());
	}
	if (singularTypeP()) {
		if (globalVerboseMode >= 2) {
			if (checkState == CHECK_HYPERACTIVE)
				strcat(buffer,"[hyper]");
		}
		strcat(buffer," (");
		strcat(buffer,stateString());
		strcat(buffer,")");
checkClusterPropertyStates(this);
	}
	else	{
		/*
		 *	Get some host info for the cluster controller
		 */
	}
	gtk_label_set_text((GtkLabel *) label,buffer);
	if (singularTypeP()) {
		/*
		 *	Its in hyperactive state, change color of the 
		 *	text.
		 */
		switch (checkState) {
		case CHECK_HYPERACTIVE:
			gtk_widget_set_style(label,cautionStyle); 
			break;
		default:
			gtk_widget_set_style(label,normalStyle); 
			break;
		}
	}
}

Node::Node(NodeType ty,char *nm,Node *tree,ConnectInfo *cn) 
{
	name		= strdup(nm);
	type		= ty;
	connection	= cn;
	state		= N_STATE_UNKNOWN;
	prop_win	= NULL;
	sel_win		= NULL;
	sel_log		= NULL;
	console_win	= NULL;
	console_zvt	= NULL;
	subtree		= NULL;
	serial_number	= NULL;
	emp_port	= NULL;
	ip		= NULL;
	password	= NULL;
	console		= NULL;
	childList	= NULL;
	pending		= NULL;
	alias		= NULL;
	checkState	= CHECK_HYPERACTIVE;
	main_list	= NULL;
	misc_flags	= N_MISC_NOTHING;
	services	= new NodeServices(this);

	/*
	 *	Set up some of the visual aspects
	 */
	if (!colorsAlloced) doColorAlloc();

	node_item	= gtk_tree_item_new_with_label(nm);
	
	GtkWidget *l = gtk_tree_item_get_label(node_item);

	//	the 'steady state' style
	normalStyle  = gtk_style_copy(gtk_widget_get_style(l));
	normalStyle->fg[GTK_STATE_NORMAL] = colorBlack;

	//	the 'in transition' style
	cautionStyle = gtk_style_copy(gtk_widget_get_style(l));
	cautionStyle->fg[GTK_STATE_NORMAL] = colorYellow;

	/*
	 *	We have no idea what it is, so lets try and see
	 *	if we can find it in its parent's pending tree.
	 */
	gboolean isNewNode = FALSE;
	if ((type == TYPE_UNKNOWN_NODE) && (tree != NULL)) {
		NodeDataSquirrel *sq = tree->nodeSquirrelFromPending(name);
		type = (sq != NULL) ? sq->type : TYPE_UNKNOWN_NODE;
		if (type == TYPE_UNKNOWN_NODE) {
			/*
			 *      Should go check to see what it can do.
			 */
			if (globalVerboseMode >= 2) {
				fprintf(stderr,"Node::Node() - couldn't find type for `%s`!\n",name);
			}
			isNewNode = FALSE;
		}
		else    {
			/*
			 *      Because we found the variable in the pending
			 *      list, this means it is a new node.
			 */
			isNewNode = TRUE;
			if (sq) {
				emp_port = strdup(sq->device);
				console  = strdup(sq->console);
				password = strdup(sq->pw);
				ip       = strdup(sq->addr);
			}
		}
	}

	/*----------------------------------------------*
	 *      Now setup the correct set of services   *
	 *----------------------------------------------*/

	if (type == TYPE_UNKNOWN_NODE) {
		type = TYPE_EMP_NODE;	// default to EMP for now
	}

	updateNodeServices(isNewNode);
        if (type != TYPE_UNKNOWN_NODE && singularTypeP()) {
		if (isNewNode) {
			services->doNodeConfigure(handleNodeConfiguredMessage,
						  name,emp_port,password,ip);
		}
		else	{
			getInventory();   // get interesting info about the node
		}
        }

	if (tree) {
		/* insert at front of child list */
		tree->childList = g_slist_prepend(tree->childList,this);
	}

	/*
	 *	Set the events based upon the node type
	 */
	gtk_signal_connect(GTK_OBJECT(node_item), "button_press_event",
			   GTK_SIGNAL_FUNC (node_item_button_pressed), 
			   (gpointer) NULL);

	/*
	 *
	 */
	gtk_object_set_data(GTK_OBJECT(node_item),"item", (void *) this);
	gtk_object_set_data(GTK_OBJECT(node_item),"tree", (void *) tree);
	gtk_object_set_data(GTK_OBJECT(node_item),"mainList",(void *) mainList);
	/* Connect all GtkItem:: and GtkTreeItem:: signals */
	SIGCON(node_item,(char *) "select");
	SIGCON(node_item,(char *) "deselect");
	SIGCON(node_item,(char *) "toggle");
}

static void
removeFromChildLists(gpointer data,gpointer user_data) 
{
	Node	*n = (Node *) data;
	Node	*target = (Node *) user_data;

	if (n == NULL) return;
	if (n->singularTypeP()) return;
	n->removeChildNode(target);
}

Node::~Node() 
{
	suspendStatusChecks();
	/*
	 *	REMOVE THE NODE FROM EVERYTHING INTERESTING
	 */

	g_slist_remove_all(theNodeList,this);	// remove from node list

	/*
	 *	Remove it from child lists
	 */
	g_slist_foreach(theNodeList,removeFromChildLists,this);

	/*
	 *	Remove from the nexxus.
	 *
	 *	NOTE:	Should handle NODE_NOT_FOUND messages!
	 */
	if (connection->connected) {	// only if I am connected
		services->doNodeDelete();
	}

	/*
	 *	Remove from the gui
	 */
	gtk_tree_remove_item(GTK_TREE(((GtkWidget *) node_item)->parent),
			     node_item);

	/*
	 *	Clean up the node allocated stuff.
	 */
	free(name);
	resumeStatusChecks();
}

Node *
copyNode(Node *src,Node *tree) 
{
	Node	*tmp = (Node *) malloc(sizeof(Node));

	memcpy(tmp,src,sizeof(Node));
	tmp->node_item  = gtk_tree_item_new_with_label(tmp->name);
	tmp->alias = src;
	if (tree) {
		tree->childList = g_slist_prepend(tree->childList,tmp);
	}
	gtk_object_set_data(GTK_OBJECT(tmp->node_item),"item", (void *) tmp);
	gtk_object_set_data(GTK_OBJECT(tmp->node_item),"tree", (void *) tree);
	/* Connect all GtkItem:: and GtkTreeItem:: signals */
	SIGCON(tmp->node_item,(char *) "select");
	SIGCON(tmp->node_item,(char *) "deselect");
	SIGCON(tmp->node_item,(char *) "toggle");
	return(tmp);
}

/*----------------------------------------------------------------------*
 *			    ADD NODE DIALOG				*
 *----------------------------------------------------------------------*/

typedef	struct	addNodeDialogSquirrel_t {
	NodeType	node_type;	
	void (*callback)(void *handle,NodeType type,char *device,char *pw,
			 char *nodeid,char *ip,char *console,
			 gpointer clientData,gpointer callData);
	gpointer	clientData;
	gpointer	callData;
	GtkWidget	*dialog;
	GtkWidget	*device_label;
	GtkWidget	*device;
	GtkWidget	*console_label;
	GtkWidget	*console;
	GtkWidget	*password;
	GtkWidget	*nodeid;
	GtkWidget	*ip;
} AddNodeDialogSquirrel;

static void
gottaNodeInfoCancel(GtkWidget *,gpointer data)
{
	GtkWidget	*dialog;
	AddNodeDialogSquirrel *tmp = (AddNodeDialogSquirrel *) data;
	dialog = tmp->dialog;
	free(tmp);
        gtk_widget_destroy(dialog);
}

/*
 *	getText() -	get the textual information from a GtkEntry widget
 *			and return a blank string if it is empty.
 */

static char *
getText(GtkWidget *w)
{
	gchar *txt;
	char	*tmp;

	txt = gtk_entry_get_text((GtkEntry *) w);
	tmp = (txt != NULL && strlen(txt)) ? strdup((char *) txt) : strdup("");
	return(tmp);
}

static void
gottaNodeInfoAdd(GtkWidget *,gpointer data)
{
	GtkWidget	*dialog;
	AddNodeDialogSquirrel *tmp = (AddNodeDialogSquirrel *) data;

	dialog = tmp->dialog;

	if (tmp->callback == NULL) {
		/* Nothing to do, just clean up, nuke the dialog, and go */
		free(tmp);
		gtk_widget_destroy(dialog);
		return;
	}

	char *snodeid   = getText(tmp->nodeid);
	char *sdevice   = getText(tmp->device);
	char *sconsole	= getText(tmp->console);
	char *spassword = getText(tmp->password);
	char *snodeip   = getText(tmp->ip);

	if (globalVerboseMode >= 2) {
		fprintf(stderr,"gottaNodeInfoAdd() - nodeid=%s, device=%s, password=%s, ip=%s\n",
			snodeid,sdevice,spassword,snodeip);
	}

	/*
	 *	Do some verification
	 */

	if (!*snodeid) {
		informational_dialog("You need to specify a node name!");
		return;
	}
	
	/*
	 *	Let'r rip!
	 */

	tmp->callback((void *) tmp->clientData,tmp->node_type,sdevice,
		      spassword,snodeid,snodeip,sconsole,tmp->clientData,
		      tmp->callData);

	/*
	 *	clean up and get out
	 */
	free(tmp);
        gtk_widget_destroy(dialog);
}

static void
node_type_select(gpointer data,GtkWidget *w)
{
	AddNodeDialogSquirrel *tmp = (AddNodeDialogSquirrel *) data;

	tmp->node_type = (NodeType) ((int) gtk_object_get_data(GTK_OBJECT(w),
							       "node_type"));

	switch(tmp->node_type) {
	case TYPE_RSH_NODE:
		gtk_widget_set_sensitive(tmp->device,FALSE);
		gtk_widget_set_sensitive(tmp->device_label,FALSE);
		gtk_widget_set_sensitive(tmp->console,FALSE);
		gtk_widget_set_sensitive(tmp->console_label,FALSE);
		break;
	default:
		gtk_widget_set_sensitive(tmp->device,TRUE);
		gtk_widget_set_sensitive(tmp->device_label,TRUE);
		gtk_widget_set_sensitive(tmp->console,TRUE);
		gtk_widget_set_sensitive(tmp->console_label,TRUE);
		break;
	}
}

void 
get_nodeinfo_dialog(gchar *msg,
		    void (*callback)(void *handle,NodeType type, char *device,
				     char *pw, char *nodeid,char *ip,
				     char *console,
				     gpointer clientData,gpointer callData),
		    gpointer clientData) 
{
	GtkWidget	*device_label;
	GtkWidget	*console_label;
	AddNodeDialogSquirrel	*tmp;
	int	vPos = 0;

	tmp = (AddNodeDialogSquirrel *) malloc(sizeof(AddNodeDialogSquirrel));

	/* Create the widgets */

	GtkWidget *dialog = gtk_dialog_new();
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox),5);

	/*------------------------------------------------------*
	 *			Top portion			*
	 *------------------------------------------------------*/

        GtkWidget *hbox = gtk_hbox_new(FALSE /*TRUE*/, 3);
	gtk_box_set_spacing(GTK_BOX(hbox),10);
        gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
        gtk_widget_show (hbox);

	GtkWidget *icon = getIcon((gchar **) question_xpm);
	gtk_container_add(GTK_CONTAINER(hbox),icon);

	GtkWidget *label = gtk_label_new(msg);
	gtk_container_add(GTK_CONTAINER(hbox),label);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),hbox);

	/*------------------------------------------------------*
	 *			Data portion			*
	 *------------------------------------------------------*/

        GtkWidget *table = gtk_table_new(6 /*rows*/,2 /*cols*/,FALSE);
	gtk_table_set_row_spacings((GtkTable *) table,5);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

	/*
	 *	Node type option menu
	 */

	GtkWidget *opt_menu = gtk_menu_new();

		/*----------------------*
		 *	EMP NODE	*
		 *----------------------*/

	GtkWidget *emp_node = gtk_menu_item_new_with_label("EMP Node");
	gtk_object_set_data(GTK_OBJECT(emp_node),"node_type",
			    (gpointer) TYPE_EMP_NODE);
	gtk_signal_connect_object(GTK_OBJECT(emp_node),"activate",
				  GTK_SIGNAL_FUNC(node_type_select),
				  (GtkObject *) tmp);
	gtk_menu_append(GTK_MENU(opt_menu),emp_node);
        gtk_widget_show(emp_node);

#ifndef	EMP_ONLY
		/*----------------------*
		 *	MSC NODE	*
		 *----------------------*/

	GtkWidget *msc_node = gtk_menu_item_new_with_label("MSC Node");
	gtk_object_set_data(GTK_OBJECT(msc_node),"node_type",
			    (gpointer) TYPE_MSC_NODE);
//	gtk_widget_set_sensitive(msc_node,FALSE);
	gtk_signal_connect_object(GTK_OBJECT(msc_node),"activate",
				  GTK_SIGNAL_FUNC(node_type_select),
				  (GtkObject *) tmp);
	gtk_menu_append(GTK_MENU(opt_menu),msc_node);

		/*----------------------*
		 *	RSH NODE	*
		 *----------------------*/

	GtkWidget *rsh_node = gtk_menu_item_new_with_label("RSH Node");
	gtk_object_set_data(GTK_OBJECT(rsh_node),"node_type",
			    (gpointer) TYPE_RSH_NODE);
//	gtk_widget_set_sensitive(rsh_node,FALSE);
	gtk_signal_connect_object(GTK_OBJECT(rsh_node),"activate",
				  GTK_SIGNAL_FUNC(node_type_select),
				  (GtkObject *) tmp);
	gtk_menu_append(GTK_MENU(opt_menu),rsh_node);
#endif	/*EMP_ONLY*/

        gtk_widget_show(opt_menu);

	GtkWidget *opt = gtk_option_menu_new();
	gtk_table_attach(GTK_TABLE(table),opt,0,2,vPos,
			 vPos+1,GTK_SHRINK, GTK_SHRINK, 0,0);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(opt),opt_menu);
        gtk_widget_show(opt);
	vPos++;

		/*------------------------------*
		 *	DATA ABOUT THE NODE	*
		 *------------------------------*/

#define DIALOG_LABEL(l,s)						\
                l = gtk_label_new(s);					\
		gtk_label_set_justify(GTK_LABEL(l), GTK_JUSTIFY_LEFT);	\
                gtk_table_attach(GTK_TABLE(table),l,0,1,vPos,		\
                                 vPos+1,GTK_SHRINK, GTK_SHRINK, 5,0)

	DIALOG_LABEL(label,"Node ID");
        GtkWidget *nodeid = gtk_entry_new_with_max_length(32);
        gtk_entry_set_editable(GTK_ENTRY(nodeid), TRUE);
        gtk_table_attach_defaults(GTK_TABLE(table),nodeid,1,2,vPos,vPos+1);
        gtk_widget_show(nodeid);
        gtk_widget_show(label);
	vPos++;

	DIALOG_LABEL(device_label,"EMP Device");
        GtkWidget *device = gtk_entry_new_with_max_length(32);
        gtk_entry_set_editable(GTK_ENTRY(device), TRUE);
        gtk_table_attach_defaults(GTK_TABLE(table),device,1,2,vPos,vPos+1);
        gtk_widget_show(device);
        gtk_widget_show(device_label);
	vPos++;

	DIALOG_LABEL(console_label,"Console Device");
        GtkWidget *console = gtk_entry_new_with_max_length(32);
        gtk_entry_set_editable(GTK_ENTRY(console), TRUE);
        gtk_table_attach_defaults(GTK_TABLE(table),console,1,2,vPos,vPos+1);
        gtk_widget_show(console);
        gtk_widget_show(console_label);
	vPos++;

	DIALOG_LABEL(label,"Password");
        GtkWidget *password = gtk_entry_new_with_max_length(32);
        gtk_entry_set_editable(GTK_ENTRY(password), TRUE);
        gtk_table_attach_defaults(GTK_TABLE(table),password,1,2,vPos,vPos+1);
        gtk_widget_show(password);
        gtk_widget_show(label);
	vPos++;


	DIALOG_LABEL(label,"Node IP");
        GtkWidget *ip = gtk_entry_new_with_max_length(32);
        gtk_entry_set_editable(GTK_ENTRY(ip), TRUE);
        gtk_table_attach_defaults(GTK_TABLE(table),ip,1,2,vPos,vPos+1);
	gtk_signal_connect_object(GTK_OBJECT(ip), "activate",
		       		  GTK_SIGNAL_FUNC(gtk_window_activate_default),
				  GTK_OBJECT(dialog));
        gtk_widget_show(ip);
        gtk_widget_show(label);
	vPos++;

	/*------------------------------------------------------*
	 *			Button portion			*
	 *------------------------------------------------------*/

	/*
	 *	Stash away some stuff for later
	 */
	tmp->node_type	= TYPE_EMP_NODE;	// default for now
	tmp->dialog     = dialog;
	tmp->callback   = callback;
	tmp->clientData = clientData;
	tmp->device     = device;
	tmp->device_label = device_label;
	tmp->console	= console;
	tmp->console_label = console_label;
	tmp->password   = password;
	tmp->nodeid     = nodeid;
	tmp->ip		= ip;

	GtkWidget *add_button = gtk_button_new_with_label("Yes");
	gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
		       	   GTK_SIGNAL_FUNC (gottaNodeInfoAdd), (gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  add_button);
	GTK_WIDGET_SET_FLAGS(add_button,GTK_CAN_DEFAULT);
	gtk_widget_grab_default(add_button);

	GtkWidget *cancel_button = gtk_button_new_with_label("No");
	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
		       	   GTK_SIGNAL_FUNC(gottaNodeInfoCancel),(gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  cancel_button);

	/* Add the label, and show everything we've added to the dialog. */
	gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_MOUSE);
	gtk_widget_show_all(dialog);
}

static void
handleNodeConfiguredMessage(InputEntry * /*entry*/,gpointer clientData)
{
	Node	*node = (Node *) clientData;

	node->getInventory();
}

static void
handleNodeStatusMessage(InputEntry *entry,gpointer clientData)
{
	Node	*node = (Node *) clientData;
	char	*state;
	char	*inPtr;
	int	old_state;

	if (node == NULL) return;
	for (InputEntry *tmp = entry; tmp; tmp = tmp->next) {
		inPtr = tmp->string;
		(void) strtok(inPtr,":");	// skip CSTATUS
		state = strtok(NULL,":");	// the state
		old_state = node->getState();
		if (old_state == N_STATE_NOAUTH) {
			/*
			 *	its probably running but I can't connect,
			 *	so why bother
			 */
			node->getInventory();
			continue;
		}
		node->setState((!strcmp(state,"YES")) ? N_STATE_RUNNING
						      : N_STATE_DEAD);
		if (node->getState() == N_STATE_RUNNING) {
			node->checkState = CHECK_NORMALLY;
//			if (old_state != N_STATE_RUNNING) {
//				node->getInventory();
//			}
		}
	}
	node->updateVisual();
}

void
Node::checkStatus(gpointer data)
{
	int	state = (int) data;

	if (globalStatusChecksSuspended) return;
	if (state != checkState) return;
	if (connection == NULL)	return;
	if (!singularTypeP()) return;
	if (!strcmp(name,"NA")) {
		recheckNodeName();
		return;
	}
	services->doMachineStatus(handleNodeStatusMessage);
	checkClusterPropertyStates(NULL);
}

GtkWidget *
Node::getSubtree() 
{
	if (subtree == NULL) {
		subtree = gtk_tree_new();
		assert(subtree != NULL);
		assert(node_item != NULL);
		gtk_tree_set_view_mode(GTK_TREE(subtree),GTK_TREE_VIEW_ITEM);
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(node_item),subtree);
	}
	return(subtree);
}

/*----------------------------------------------------------------------*
 *			NODE ADD FOR CLUSTER				*
 *----------------------------------------------------------------------*/

static void
handleNodeAddMessage(InputEntry * /*entry*/,gpointer clientData)
{
	NodeDataSquirrel *sq = (NodeDataSquirrel *) clientData;

	/* NOTE: should probably check entries for success! */

	/*--------------------------------------*
	 *	add to pending node list	*
	 *--------------------------------------*/

	if (globalVerboseMode >= 2) {
		fprintf(stderr,"###add pending node '%s' to '%s' (type=%d)\n",
			sq->nodeid,sq->cluster->name,(int) sq->type);
	}
	sq->cluster->pending = g_slist_prepend(sq->cluster->pending,sq);

        sq->cluster->connection->refreshNodeList();     // should only be one
}

static void
doTheNodeAdd(void *data,NodeType type, char *device,char *pw,char *nodeid,
	     char *ip,char *console, gpointer /*clientData*/,
	     gpointer /*callData*/)
{
	Node	*cluster = (Node *) data;

	NodeDataSquirrel *tmp = new NodeDataSquirrel(cluster,type,nodeid,device,pw,ip,console);
	cluster->services->doNodeAdd(handleNodeAddMessage,type,nodeid,(gpointer) tmp);
}

void
Node::getNodeForCluster()
{
        char    buf[1024];

        sprintf(buf,"Add node to cluster  \n\"%s\"",connection->hostname);
	get_nodeinfo_dialog(buf,doTheNodeAdd,(gpointer) this);
}

/*----------------------------------------------------------------------*
 *			NODE INVENTORY STUFF				*
 *----------------------------------------------------------------------*/

#include <netdb.h>
#ifndef	__sgi
#include <netinet/in.h>
#endif	/*__sgi*/
#include <sys/socket.h>
#include "vacmclient_api.h"

extern "C" {
extern char *inet_ntoa(struct in_addr in);
}

static void
handleNodeInventoryMessage(InputEntry *entry,gpointer clientData)
{
        Node    *node = (Node *) clientData;
        node->services->handleInventoryMessage(entry);
}

int
Node::getInventory2()
{
	if (type == TYPE_CLUSTER) {
		struct in_addr ip_addr;
		/*
		 *	Just do some local info
		 */
		if (api_nexxus_return_ip(connection->handle,&ip_addr)==API_RETURN_OK) {
			char *dot = strdup((char *) inet_ntoa(ip_addr));
//			fprintf(stderr,"dot=%s strlen(dot)=%d\n",dot,strlen(dot));
			struct hostent *myHostEnt = gethostbyname(dot);
			if (myHostEnt == NULL) {
				herror("getting hostbyaddr");
				return(0);
			}
		}
		return(1);
	}
	services->doNodeInventory(handleNodeInventoryMessage);
	return(0);
}

static void
handleNodeInquiryMessage(InputEntry *entry,gpointer clientData)
{
	Node    *node = (Node *) clientData;
	node->services->handleInquiryMessage(entry);
	/*
	 *	Now, get the other stuff
	 */
	node->getInventory2();
}

int
Node::getInventory()
{
	services->doNodeInquiry(handleNodeInquiryMessage);
	return(0);
}

/*----------------------------------------------------------------------*
 *			     NODE PENDING STUFF				*
 *----------------------------------------------------------------------*/

NodeType
Node::nodeTypeFromPending(char *nodeid)
{
	NodeDataSquirrel *sq;

	if (nodeid == NULL) return(TYPE_UNKNOWN_NODE);
	for (unsigned int i = 0; i < g_slist_length(pending); i++) {
		sq = (NodeDataSquirrel *) g_list_nth_data((GList *) pending,i);
		if (sq->nodeid && !strcmp(sq->nodeid,nodeid)) {
			return(sq->type);
		}
	}
	return(TYPE_UNKNOWN_NODE);
}

NodeDataSquirrel *
Node::nodeSquirrelFromPending(char *nodeid)
{
	NodeDataSquirrel *sq;

	if (nodeid == NULL) return(NULL);
	for (unsigned int i = 0; i < g_slist_length(pending); i++) {
		sq = (NodeDataSquirrel *) g_list_nth_data((GList *) pending,i);
		if (sq->nodeid && !strcmp(sq->nodeid,nodeid)) {
			return(sq);
		}
	}
	return(NULL);
}

void    
Node::doSetVariable(char *var,char *value)
{
	char	msgPacket[1024];
	sprintf(msgPacket,"NEXXUS:SET_VAR:%s:%s:%s:",name,var,value);
	connection->sendMsg(msgPacket,FALSE,NULL,NULL);
}

/*----------------------------------------------------------------------*
 *				NODE TYPE DISCOVERY			*
 *----------------------------------------------------------------------*/

static void
handleNodeDiscoveryMessage(InputEntry *entry,gpointer clientData)
{
        Node	*node = (Node *) clientData;

        for (InputEntry *tmp = entry; tmp; tmp = tmp->next) {
		if (strncmp(tmp->string,"GET_VAR",7)) continue;
		char *cp = tmp->string + strlen("GET_VAR:");
		NodeType nT = nodeTypeFromName(cp);
		if (nT == TYPE_UNKNOWN_NODE) continue;
		node->type = nT;
		node->updateNodeServices(FALSE);
		node->updateVisual();
		node->getInventory();   // get interesting node info
		return;
	}
}

void
Node::typeDiscovery()
{
	char	msgPacket[1024];
	sprintf(msgPacket,"NEXXUS:GET_VAR:%s:TYPE:",name);
	connection->sendMsg(msgPacket,TRUE,handleNodeDiscoveryMessage,this);
}

void
Node::updateNodeServices(gboolean isNewNode) 
{
	if (globalVerboseMode >= 2) {
		fprintf(stderr,"creating %s node '%s'\n",nodeNameFromType(type),
			name);
	}
	switch (type) {
#ifdef	notdef
	/*
	 *	For now, just default to EMP
	 */
	case TYPE_UNKNOWN_NODE: // we have no idea, time for discovery
		typeDiscovery();
		return;
#endif	/*notdef*/
	case TYPE_MSC_NODE:
		services = new MSCservices(this);
		if (isNewNode) {
			doSetVariable("TYPE","MSC");
		}
		break;
	case TYPE_RSH_NODE:
		services = new RSHservices(this);
		if (isNewNode) {
			doSetVariable("TYPE","RSH");
		}
		break;
	default:        // everything else means EMPnode
		services = new EMPservices(this);
		if (isNewNode) {
			doSetVariable("TYPE","EMP");
		}
		break;
	}
}

