/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

#ifndef	NODE_H
#define	NODE_H

class Node;
class NodeServices;

#include <stdio.h>
#include <stdlib.h>
#include "hoover.h"
#include "connection.h"

typedef void (*operFunc)(Node *,gpointer data);

extern GSList	*g_slist_remove_all(GSList *,void*);

#define RESET_IT	1
#define POWER_DOWN	2
#define POWER_UP	3
#define VIEW_LOG	4
#define SERIAL_CONSOLE	5
#define PROPERTIES	6
#define STATUS		7
#define REMOVE_IT	8
#define SENSE_IT	9

#define	CHECK_HYPERACTIVE	0
#define	CHECK_NORMALLY		1
#define	CHECK_SLOWLY		2

typedef enum {
	N_STATE_UNKNOWN,
	N_STATE_CONNECTING,
	N_STATE_NEGOTIATING,
	N_STATE_WAITING,
	N_STATE_DEAD,
	N_STATE_RUNNING,
	N_STATE_INITIALIZING,
	N_STATE_BOOTING,
	N_STATE_NOAUTH,
} NodeState;

class NodeDataSquirrel {
public:
	NodeDataSquirrel(Node *c,NodeType t,char *nid,char *dev,char *pass,char *ip,char *con) {
		cluster	= c;
		type	= t;
		nodeid	= strdup(nid  ? nid  : "");
		device	= strdup(dev  ? dev  : "");
		console	= strdup(con  ? con  : "");
		pw	= strdup(pass ? pass : "");
		addr	= strdup(ip   ? ip   : "");
	}
	~NodeDataSquirrel() {
		free(nodeid);
		free(device);
		free(pw);
		free(addr);
	}
	Node	*cluster;
	NodeType type;
	char	*nodeid;
	char	*device;
	char	*console;
	char	*pw;
	char	*addr;
};


class Node {
protected:
	GtkStyle	*normalStyle;
	GtkStyle	*cautionStyle;
	GtkWidget	*subtree;
	NodeState	state;
public:
	Node() {};
	Node(NodeType ty = TYPE_EMP_NODE,char *nm = NULL,Node *tree = (Node *) NULL,ConnectInfo *cn = (ConnectInfo *) NULL);
	~Node();
	ConnectInfo	*connection;
        gchar	*typeName()             { return("Node");    }
	void	typeDiscovery();
	void	updateVisual();
	void	print();
	void	getNodeForCluster();
	int	getInventory();		// find interesting info about node
	int	getInventory2();	// find interesting info about node
	NodeType nodeTypeFromPending(char *nodeid);
	NodeDataSquirrel *nodeSquirrelFromPending(char *nodeid);
	char	*nodeTypeName()	{ return(nodeNameFromType(type));	}
	void	updateNodeServices(gboolean isNewNode);

	gboolean singularTypeP() {
		return(	  type == TYPE_EMP_NODE 
		       || type == TYPE_MSC_NODE
		       || type == TYPE_RSH_NODE);
	}
	int	numChildren() {
		return(singularTypeP() ? 1 : g_slist_length(childList));
	}
	void	removeChildNode(Node *target) {
        	childList = g_slist_remove_all(childList,target);
	}
	void	addToSubtree(GtkWidget *w) {
		gtk_tree_append(GTK_TREE(getSubtree()),w);
	}
	void	clearSubtree() {
		gtk_tree_clear_items((GtkTree *) getSubtree(),0,-1);
	}
	void	expandSubtree() {
		gtk_tree_item_expand((GtkTreeItem *) node_item);
	}
	void	collapseSubtree() {
		gtk_tree_item_collapse((GtkTreeItem *) node_item);
	}
	GtkWidget *getSubtree();
	void recheckNodeName() {
		if (connection->handle != NULL)	{
			connection->refreshNodeList();
		}
	}
	char	*stateString();
	NodeState	getState() 	{ return(state);	}
	void	setState(NodeState st)	{ state = st; 		}
	void	checkStatus(gpointer data);
	void	mapFunc(operFunc func,gpointer data = NULL) {
		if (func == NULL) return;
		if (type == TYPE_CLUSTER || type == TYPE_METACLUSTER) {
			for (unsigned int i = 0; i < g_slist_length(childList); i++) {
				Node *n = (Node *) g_list_nth_data((GList *) childList,i);
				func(n,data);
			}
		}
		else    {
			func(this,data);
		}
	}

	/*
	 *
	 */

	char	*name;			// the node id
	NodeType	type;		// node type (i.e. cluster, EMP, etc.
	short		checkState;
	GtkWidget	*node_item;
	GtkWidget	*prop_win;	// properties window
	GtkWidget	*sel_win;	// event log window
	GtkWidget	*sel_log;	// event log list & cluster prop clist
	GtkWidget	*console_win;	// serial console window
	GtkWidget	*console_zvt;	// serial console 

	GtkWidget	*main_list;	// ctree that the node is in

	char	*serial_number;
	char	*emp_port;
	char	*console;
	char	*ip;
	char	*password;
	GSList	*childList;	/* list of children	*/
	Node	*alias;		/* the real node	*/
	GSList	*pending;	/* pending node list	*/

	/*
	 *	These are miscellaneous transient flags
	 */
#define	N_MISC_NOTHING		0x0000
#define	N_MISC_TRAVERSED	0x0001
	unsigned int	misc_flags;

	/*
	 *
	 */
	void	doSetVariable(char *,char *);

	NodeServices	*services;
};

/*----------------------------------------------------------------------*
 *			     BOGUS NODE SERVICES			*
 *----------------------------------------------------------------------*/

class NodeServices {
protected:
	char	msgPacket[256];		// common message buffer
        void	sendMsg(char *cmd,gboolean atCompletion,InputEntryProc theFunc,
			gpointer cd) {
		parent->connection->sendMsg(cmd,atCompletion,theFunc,cd);
	}

public:
	Node	*parent;
	NodeServices()		{ /* BOGUS!!!! */	}
	NodeServices(Node *p)	{ parent = p;		}
	char	*typeName()	{ return("Generic Node Services");	}

	/*
	 *	Node Actions
	 */
	void doNodeAdd(InputEntryProc func,NodeType /*type*/,
		       char *nodeid,gpointer clientData) {
                sprintf(msgPacket,"NEXXUS:NODE_ADD:%s:default",nodeid);
                sendMsg(msgPacket,TRUE,func,(gpointer) clientData);
	}
	void doNodeDelete() {
		sprintf(msgPacket,"NEXXUS:NODE_DEL:%s:",parent->name);
		sendMsg(msgPacket,TRUE,(InputEntryProc) NULL,(gpointer) parent->connection);
	}
	virtual void doNodeConfigure(InputEntryProc /*func*/,char * /*nodeid*/,
                       		     char * /*device*/,char * /*pw*/,char * /*addr*/) {
#ifdef	notdef
		/*
		 *	NOTE!!!!! This is the wrong place for this, but
		 *	right now I don't have the pending node add working,
		 *	so I assume everything is an EMP node.
		 */
		sprintf(msgPacket,"emp:CONFIGURATION:%s:%s:%s:",nodeid,device,(pw && *pw != '\0') ? pw : "NONE");
		sendMsg(msgPacket,TRUE,func,(gpointer) parent);
#else	/*notdef*/
		fprintf(stderr,"Node::doNodeConfigure() unimplemented!\n");
#endif	/*notdef*/
                doConsoleConfigure(NULL,NULL);
	}
	virtual void doMachineStatus(InputEntryProc /*func*/) {
		fprintf(stderr,"Node::doMachineStatus() unimplemented!\n");
	}
	virtual void doNodeInventory(InputEntryProc /*func*/) {
		fprintf(stderr,"Node::doNodeInventory() unimplemented!\n");
	}
	virtual void doNodeInquiry(InputEntryProc /*func*/) {
		fprintf(stderr,"Node::doNodeInquiry(%s) unimplemented!\n",parent->name);
	}
	virtual void getSENThresholds(char * /*sensor*/,gpointer/*panel*/,InputEntryProc /*func*/) {
		fprintf(stderr,"Node::getSENThresholds() unimplemented!\n");
	}
        virtual void doConsoleConfigure(gpointer/*panel*/,InputEntryProc /*func*/) {
		fprintf(stderr,"Node::doConsoleConfigure() unimplemented!\n");
	}
	virtual void doConsoleActivate(gpointer/*panel*/,InputEntryProc /*func*/) {
		fprintf(stderr,"Node::doConsoleActivate() unimplemented!\n");
	}
	virtual void doConsoleSteal(gpointer/*panel*/,InputEntryProc /*func*/) {
		fprintf(stderr,"Node::doConsoleSteal() unimplemented!\n");
	}
	virtual void doConsoleDisconnect() {
		fprintf(stderr,"Node::doConsoleDisconnect() unimplemented!\n");
	}
	virtual void doConsoleAlerts(gpointer/*panel*/,InputEntryProc /*func*/) {
		fprintf(stderr,"Node::doConsoleAlerts() unimplemented!\n");
	}
	virtual void doSENLogging(gpointer/*panel*/,InputEntryProc /*func*/) {
		fprintf(stderr,"Node::doSENLogging() unimplemented!\n");
	}
	virtual void doGetSensorValue(gpointer/*panel*/,InputEntryProc /*func*/,int /*sensor*/) {
		fprintf(stderr,"Node::doGetSensorValue() unimplemented!\n");
	}
	virtual void doSELLogging(gpointer/*panel*/,InputEntryProc /*func*/) {
		fprintf(stderr,"Node::doSELLogging() unimplemented!\n");
	}
	virtual void doReset()	{ 
		fprintf(stderr,"Node::doReset() unimplemented!\n");
	}
	virtual void doPowerUp()	{ 
		fprintf(stderr,"Node::doPowerUp() unimplemented!\n");
	}
	virtual void doPowerDown()	{ 
		fprintf(stderr,"Node::doPowerDown() unimplemented!\n");
	}
	virtual void doStimulateSELEntry(int /*n*/) {
		fprintf(stderr,"Node::doStimulateSELEntry() unimplemented!\n");
	}
	virtual void doSetNodeType() {
		fprintf(stderr,"Node::doSetNodeType() unimplemented!\n");
	}
	virtual void handleInventoryMessage(InputEntry * /*entry*/) {
		fprintf(stderr,"Node::handleInventoryMsg() unimplemented!\n");
	}
	virtual void handleInquiryMessage(InputEntry * /*entry*/) {
		fprintf(stderr,"Node::handleInquiryMsg() unimplemented!\n");
	}
};

extern Node	*copyNode(Node *src,Node *tree);
extern NodeState stateStringToEnum(char *str);
extern char	*stateEnumToString(NodeState state);
extern void	doNodeAction(gpointer data,GtkWidget *w);

#endif	/*NODE_H*/
