/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 * This program is distributed under the GNU General Public License (GPL)
 * as outlined in the COPYING file.
 *
 * Copyright (C) 1999, San Mehat, and VA Linux Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 *
 */

#include <stdio.h>
#include <sys/socket.h>
#include <signal.h>
#include <netinet/in.h>
#include <sys/un.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#include <linux/ipmi_ioctls.h>

void signal_handler(int I_signo);
static int alloc_nexxus_slot(void);
static void become_daemon(void);
extern int errno;

int	nexxus_cons[255];

typedef struct sensord_req
	{
	unsigned char	sensor_number;
	unsigned char	identify_cmd;
	} EXTENTD_REQ;

typedef struct sensord_rep
	{
	char					key;
	char					ext_major_ver;
	char					ext_minor_ver;
	char					drv_major_ver;
	char					drv_minor_ver;
	__uint32_t		rc;
	__uint32_t		datalen;
	unsigned char	cc;
	unsigned char	buff[4];
	} EXTENTD_REP;

static int handle_sensor_request(int slot, EXTENTD_REQ *request);
int check_old_ipmi(void);
int check_new_ipmi(void);
int set_drv_ver(void);

static int	ipmi_fd;
static char	ext_major_ver = 4;
static char	ext_minor_ver = 0;
static char	drv_major_ver = 0;
static char	drv_minor_ver	= 0;

int set_drv_ver()
{
	FILE	*fp;
	char	line[255];
	char	*bword;

	if (!(fp = fopen("/proc/ipmi_kcs","ro")))
		return(-1);

	fgets(line, sizeof(line), fp);
	line[(strlen(line)-1)]=0;

	bword = strtok(line, ":");
	bword = strtok(NULL, ":");

	sscanf(bword,"%d.%d",(int *) &drv_major_ver, (int *) &drv_minor_ver);
	printf("Detected driver version %d.%d\n",drv_major_ver,drv_minor_ver);
	fclose(fp);
	return(0);
}

int main(argc, argv)
int argc;
char **argv;
{
	struct	sockaddr_in	bind_sock, addr_in;
	int 								rc, i, I_sockfd = 0, max;
	fd_set							read_fds;
	struct timeval			to;
	EXTENTD_REQ					request_pkt;

	if (argc != 1)
		{
		printf("This version of the EMP Extensions Daemon does not take arguments\n");
		printf("Please update your script for this version (%d.%d)\n",
						ext_major_ver, 
						ext_minor_ver);
		exit(1);
		}

	for (i=0;i<255;i++)
		nexxus_cons[i] = -1;

	printf("VACM EMP Extensions Daemon v%d.%d (c) 2000 San Mehat (nettwerk@valinux.com)\n",
					ext_major_ver,
					ext_minor_ver);
	signal(SIGPIPE, SIG_IGN);

	if (check_old_ipmi()<0)
		{
		printf("Error: Old IPMI driver detected\n");
		printf("       This utility requires the IPMI KCS 2.0 driver available\n");
		printf("       at http://ipmitools.sourceforge.net\n");
		printf("       If you have already installed this package, make sure\n");
		printf("       you have properly created the /dev/ipmi/kcs device\n");
		exit(1);
		}
	if (check_new_ipmi()<0)
		{
		printf("Error: IPMI driver not detected\n");
		printf("       This utility requires the IPMI KCS 2.0 driver available\n");
		printf("       at http://ipmitools.sourceforge.net\n");
		printf("       If you have already installed this package, make sure\n");
		printf("       you have properly created the /dev/ipmi/kcs device\n");
		exit(1);
		}
	if ((ipmi_fd=open("/dev/ipmi/kcs",O_RDWR))<0)
		{
		fprintf(stderr,"Unable to open /dev/ipmi/kcs (%m)\n");
		exit(1);
		}
	/* Set the driver version flags */
	if (set_drv_ver()<0)
		{
		printf("Error: Unable to detect driver version\n");
		printf("       Make sure /proc/ipmi_kcs exists\n");
		exit(1);
		}
	become_daemon();

 	if ((I_sockfd = socket(AF_INET, SOCK_STREAM,0))<0)
   	{
		perror("socket");
		exit(1);
   	}
 	memset(&bind_sock,0,sizeof(bind_sock));
 	bind_sock.sin_family= AF_INET;
 	bind_sock.sin_addr.s_addr= htonl(INADDR_ANY);
 	bind_sock.sin_port= htons(9352);

 	if (bind(I_sockfd,
         	(struct sockaddr *) &bind_sock,
         	sizeof(bind_sock))<0) 
   	{
		perror("bind");
		exit(1);
   	}
	if (listen(I_sockfd, 5)<0)
		{
		perror("listen");
		exit(1);
		}
	/* Main loop */
	while(1)
		{
		FD_ZERO(&read_fds);
		to.tv_sec=10;
		to.tv_usec=0;

		max = 0;
		FD_SET(I_sockfd, &read_fds);
		max = I_sockfd;
		for (i=0; i < 255; i++)
			{
			if (nexxus_cons[i] != -1)
				{
				FD_SET(nexxus_cons[i], &read_fds);
				if (nexxus_cons[i] > max)
					max = nexxus_cons[i];
				}
			}
		if ((rc = select((max+1), &read_fds, NULL, NULL, &to))<0)
			{
			perror("select");
			continue;
			}
		if (rc == 0)
			{
			continue;
			}
		if (FD_ISSET(I_sockfd, &read_fds))
			{
			int	slot, len;
		
			if ((slot = alloc_nexxus_slot())<0)
				{
				fprintf(stderr,"No nexxus slots available\n");
				continue;
				}
			if ((nexxus_cons[slot] = accept(I_sockfd,
																			(struct sockaddr *)&addr_in,
																			&len))<0)
				{
				fprintf(stderr,"Unable to accept connection from nexxus (%m)\n");
				nexxus_cons[slot] = -1;
				continue;
				}
			}
		for (i=0;i<255;i++)
			{
			if(nexxus_cons[i] != -1)
				{
				if (FD_ISSET(nexxus_cons[i], &read_fds))
					{
					if ((rc =read(nexxus_cons[i],
									(void *) &request_pkt,
									sizeof(request_pkt)))!=sizeof(request_pkt))
						{
						if (rc == 0)
							{
							close(nexxus_cons[i]);
							nexxus_cons[i] = -1;
							continue;
							}
						if (rc <0)
							{
							fprintf(stderr,"read from nexxus failed (%m)\n");
							close(nexxus_cons[i]);
							nexxus_cons[i] = -1;
							continue;
							}
						fprintf(stderr,"Split packet from nexxus detected\n");
						close(nexxus_cons[i]);
						nexxus_cons[i] = -1;
						continue;
						}
					/* OK pkt */
					handle_sensor_request(i, &request_pkt);
					}
				}
			}
		}
	close(ipmi_fd);
	exit(0);
}

static int handle_sensor_request(int slot, EXTENTD_REQ *request)
{
	IPMI_XFER			xfer;
	IPMI_XFER			*xferp = &xfer;
	unsigned char	sensor_number;
	EXTENTD_REP		response_pkt;
	int						rc;
	int						len;

	memset(&response_pkt, 0, sizeof(response_pkt));
	response_pkt.key = 0x69;

	sensor_number = request->sensor_number;
	if (!request->identify_cmd)
		{
		INIT_XFER(xferp);
		SET_REQUEST_LUN(xferp, 0);
		SET_REQUEST_NETFN(xferp, SENSOR_REQUEST);
		SET_REQUEST_CMD(xferp, 0x2d);
		SET_REQUEST_DATA(xferp, &sensor_number, sizeof(sensor_number));
		
		rc=ioctl(ipmi_fd, IOCTL_IPMI_XFER, (void *) &xfer);
		if (!rc)
			{
			GET_RESPONSE_CC(xferp, response_pkt.cc);
			GET_RESPONSE_DATA_LENGTH(xferp, len);
			response_pkt.datalen = htonl(len);
			response_pkt.rc = htonl(0);
			}
		if (rc < 0)
			{
			response_pkt.rc = htonl(errno);
			response_pkt.cc = 0;
			}
		GET_RESPONSE_DATA(xferp, &response_pkt.buff);
		}
	else
		{
		INIT_XFER(xferp);
		SET_REQUEST_LUN(xferp, 0);
		SET_REQUEST_NETFN(xferp, APP_REQUEST);

		if (request->sensor_number == 1)
			{
			SET_REQUEST_CMD(xferp, 0x55);
			}
		else
			{
			SET_REQUEST_CMD(xferp, 0x56);
			}
		rc=ioctl(ipmi_fd, IOCTL_IPMI_XFER, (void *) &xfer);
		response_pkt.datalen = 0;
		if (rc < 0)
			response_pkt.rc = htonl(errno);
		else
			response_pkt.rc = htonl(0);
		}

#if 0
printf("Response pkt len -> %d, rc = %d, cc = 0x%.2x\n",
				ntohl(response_pkt.datalen),
				ntohl(response_pkt.rc),
				response_pkt.cc);
#endif

	if (write(nexxus_cons[slot],
						(void *) &response_pkt,
						sizeof(response_pkt))<0)
		{
		perror("write");
		return(-1);
		}
	return(0);
}

static int alloc_nexxus_slot()
{
	int	i;

	for (i=0;i<255;i++)
		if (nexxus_cons[i] == -1)
			return(i);
	return(-1);
}

static void become_daemon()
{ 
  int i;
  int fd;
  
  chdir("/");
  if (fork())
    _exit(0);
  setsid(); 
  close(0); 
  close(1); 
  close(2); 
  for (i=0; i < 3; i++)
    {
    fd = open("/dev/null", O_RDWR, 0);
    if (fd < 0) 
      fd = open("/dev/null", O_WRONLY,0);
    if (fd < 0)
      {
      fprintf(stderr,"Unable to detach\n");
      _exit(1);
      }
    if (fd != i)
      {
      fprintf(stderr,"Unable to detach\n");
      _exit(1);
      }
    } 
}   

// Check for the old /dev/ipmi driver
// returns 0 if its not there (a good thing)
// -1 if if's there 
int check_old_ipmi(void)
{
	struct stat	statbuff;
	int					rc;


	if ((rc = stat("/dev/ipmi",&statbuff))<0)
		{
		if (errno == ENOENT)
			return(0);
		}
	if (S_ISDIR(statbuff.st_mode))
		return(0);
	return(-1);
}

// Check for the new /dev/ipmi/kcs driver
// returns 0 if it's there
// returns -1 if it's not there
int check_new_ipmi(void)
{
	struct stat	statbuff;
	int					rc;
	
	if ((rc = stat("/dev/ipmi",&statbuff))<0)
		return(-1);
	if (!S_ISDIR(statbuff.st_mode))
		return(-1);
	if ((rc = stat("/dev/ipmi/kcs",&statbuff))<0)
		return(-1);
	if (!S_ISCHR(statbuff.st_mode))
		return(-1);

	/* Okay the device is there.. */
	return(0);
}
