/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 *
 * Copyright (C) 2001, San Mehat, Zac Sprackett and VA Linux Systems
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.  Redistributions
 * in binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * Neither the name of the VA Linux Systems, Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <libloose.h>
#include <skel_module_harness.h>
#include <config.h>
#include <prototypes.h>
#include <signal.h>

/* This number determines the number of nodes that a slave process may control*/
#warning num_nodes_per_slave has not been adjusted to suit
int	num_nodes_per_slave = 128;

static void     signal_handler(int I_signo);
static void			config_cb(__uint32_t job_id, int client_fd, char *msg);
static void			deinit_cb(__uint32_t job_id, int client_fd, char *msg);
static void			discon_cb(__uint32_t job_id, int client_fd, char *msg);
static void			ipc_cb(char				*node_id,
											 __uint32_t	job_id,
											 int				client_fd,
											 char				*msg);
static NODELIST	*node_create(char *node_id);
static NODELIST	*lookup_node(char *node_id);

extern int errno;

NODELIST	*nodelist_head = NULL;
NODELIST	*nodelist_tail = NULL;

int main(argc, argv)
int		argc;
char	**argv;
{
	KNOWN_UNUSED_PARAM(argc);
	KNOWN_UNUSED_PARAM(argv);

  signal (SIGSEGV, signal_handler);

	lm_init();
	lm_nexxus_connect();
#error lm_register call has not been adjusted to reflect module
	if (lm_register("SKEL",
									"Skeleton Framework Module",
									"Module to be used as a template for threaded loose modules",
									"San Mehat (nettwerk@valinux.com)",
									"SKEL",
									2, 
									0) < 0) 
		{
		printf("Unable to register module with Nexxus\n");
		exit(-1);
		}

	lm_main_loop(config_cb, deinit_cb, discon_cb, ipc_cb);
	exit(0);
}

static void config_cb(__uint32_t job_id, int client_fd, char *msg)
{
	NODELIST	*node;
	char			*cmd_module;
	char			*node_id;
	char			logmsg[255];
	char			saved_msg[1024];

	KNOWN_UNUSED_PARAM(job_id);
	KNOWN_UNUSED_PARAM(client_fd);

	strcpy(saved_msg,msg);

	cmd_module = strtok(msg, ":");
	node_id = strtok(NULL, ":");

	node = lookup_node(node_id);

	if (!strcasecmp(cmd_module, "NODE"))
		{
		node = node_create(node_id);
		slave_assign_and_notify(node);
		}
	else if (!strcasecmp(cmd_module, "END_NODE"))
		{
		}
	else if (!strcasecmp(cmd_module, "ADDITION"))
		{
		node = node_create(node_id);
		slave_assign_and_notify(node);
		return;
		}
	else if (!strcasecmp(cmd_module, "DELETION"))
		{
		slave_notify_del_node(node);
		node_destroy(node);
		return;
		}
	else if (!strcasecmp(cmd_module, "RENAME"))
		{
		char	*new_name;
		
		if (!(new_name = strtok(NULL,":")))
			{
			lm_log("Malformatted cfg msg\n");
			return;
			}
		slave_notify_rename_node(node, new_name);
		strcpy(node->node_id, new_name);
		return;
		}
	else if (!strcasecmp(cmd_module, "GLOBAL"))
		{
		char	*variable;
		char	*value;
		
		if (!(variable = strtok(NULL,":")))
			{
			lm_log("Malformatted cfg msg\n");
			return;
			}
		/* BEGIN San mod 07/12/00 */
		value = &saved_msg[((variable - msg)+(strlen(variable)+1))];
		/* END San mod 07/12/00 */

#error Global variable processing has not been adjusted for module
		/* Any global variables we care about here... ie: */
		if (!strcasecmp(variable, "IP_ADDRESS"))
			{
			slave_notify_variable(node, variable, value);
			return;
			}
		}
	else if (!strcasecmp(cmd_module, "SKEL"))
		{
#error Local variable processing module name not adjusted for module
		char	*variable;
		char	*value;
		
		if (!(variable = strtok(NULL,":")))
			{
			lm_log("Malformatted cfg msg");
			return;
			}

		/* BEGIN San mod 07/12/00 */
		value = &saved_msg[((variable - msg)+(strlen(variable)+1))];
		/* END San mod 07/12/00 */

#error Local variable processing not adjusted for module
		/* Process any local variables here */
		if (!strcasecmp(variable, "DEVICE_ADDRESS"))
			{
			slave_notify_variable(node, variable, value);
			return;
			}
		else if (!strcasecmp(variable, "BAUDRATE"))
			{
			slave_notify_variable(node, variable, value);
			return;
			}
		sprintf(logmsg,
						"Received unknown local module variable '%s'",
						variable);
		lm_log(logmsg);
		}
	else
		{
		sprintf(logmsg,"Received unknown config event <%s>",cmd_module);
		lm_log(logmsg);
		}
}

static void deinit_cb(__uint32_t job_id, int client_fd, char *msg)
{
	KNOWN_UNUSED_PARAM(job_id);
	KNOWN_UNUSED_PARAM(client_fd);
	KNOWN_UNUSED_PARAM(msg);

	slave_deinit();
	lm_nexxus_disconnect();
	exit(0);
}

static void discon_cb(__uint32_t job_id, int client_fd, char *msg)
{
	slave_notify_disconnection(client_fd);
	KNOWN_UNUSED_PARAM(job_id);
  KNOWN_UNUSED_PARAM(client_fd);
	KNOWN_UNUSED_PARAM(msg);
}

static void ipc_cb(char *node_id, __uint32_t job_id, int client_fd, char *msg)
{
	char			*cmd, *p, *toss;
	char			response_msg[255], saved_msg[255];
	char			slave_msg[255];
	NODELIST	*node;

	strncpy(saved_msg,msg,(sizeof(saved_msg)-1));

	if (!(cmd = strtok(msg, ":")))
		goto out_malformatted;

	if (!(toss = strtok(NULL, ":")))
		goto out_malformatted;

	if (!(node = lookup_node(node_id)))
		{
		sprintf(response_msg,
						"%d:%d:FOR:JOB_ERROR:NODE_NOT_FOUND",
						job_id,
						client_fd);
		lm_send_to_nexxus(response_msg);
		return;
		}
	if (!node->slave)
		{
		sprintf(response_msg, "Slave not assigned for node %s",node->node_id);
		lm_log(response_msg);
		sprintf(response_msg,
						"%d:%d:FOR:JOB_ERROR:NODE_NOT_READY",
						job_id,
						client_fd);
		lm_send_to_nexxus(response_msg);
		return;
		}

	p = &saved_msg[((toss - msg) + strlen(toss)+1)];
	sprintf(slave_msg,
					"NODE_IPC:%s:%d:%d:%s:%s",
					node->node_id,
					job_id,
					client_fd,
					cmd,
					p);
	slave_send(node->slave, slave_msg);
	return;

out_malformatted:
	sprintf(response_msg,
					"%d:%d:FOR:JOB_ERROR:MALFORMATTED_MESSAGE",
					job_id,
					client_fd);
	lm_send_to_nexxus(response_msg);
	return;
}

static NODELIST	*node_create(char *node_id)
{
	NODELIST	*new;

	if (!(new = (NODELIST	*) malloc(sizeof(NODELIST))))
		return(0);
	memset(new, 0, sizeof(NODELIST));
	if (!nodelist_head)
		nodelist_head = nodelist_tail = new;
	else
		{
		nodelist_tail->next = new;
		new->prev = nodelist_tail;
		nodelist_tail = new;
		}
	strcpy(new->node_id, node_id);
	return(new);
}

static NODELIST	*lookup_node(char *node_id)
{
	NODELIST	*scan;

	if (!node_id)
		return(NULL);
	scan = nodelist_head;
	while(scan)
		{
		if (!strcmp(node_id, scan->node_id))
			return(scan);
		scan = scan->next;
		}
	return(NULL);
}

void node_destroy(NODELIST	*node)
{
  if (node == nodelist_head)
    {
    nodelist_head = nodelist_head->next;
    if (nodelist_head)
      nodelist_head->prev = NULL;
    else
      nodelist_tail = NULL;
    } 
  else if (node == nodelist_tail)
    {
    nodelist_tail = nodelist_tail->prev;
    if (nodelist_tail)
      nodelist_tail->next = NULL;
    else
      nodelist_head = NULL;
    } 
  else
    {
    node->next->prev = node->prev;
    node->prev->next = node->next;
    }
  free(node);
}

static void signal_handler(int I_signo)
{
  extern CONTEXT          *context_head;
  CONTEXT *ctx = context_head;
	char			logmsg[255];
	
  if (I_signo==SIGSEGV)
    {
		while(ctx)
			{
			if (ctx->thread == pthread_self())
				{	
				sprintf(logmsg,
						"Slave thread for node %s has crashed",
						ctx->node_id);
				lm_log(logmsg);
				sleep(5);
				abort();
				}
			ctx = ctx->next;
		  }
    }
}
