/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 *
 * Copyright (C) 2001, San Mehat, Zac Sprackett and VA Linux Systems
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.  Redistributions
 * in binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * Neither the name of the VA Linux Systems, Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <pthread.h>
#include <libloose.h>
#include <skel_module_harness.h>
#include <skel_slave_context.h>
#include <config.h>
#include <prototypes.h>

static int read_message(int, char **, __uint32_t *);
static void process_msg_from_master(int, int);
static void process_msg_from_thread(CONTEXT *, int, int);
static int send_message(int,char *);

extern CONTEXT *context_head;
extern CONTEXT *context_tail;

#define SLAVE_DEBUG
void slave_machine_start(int read_fd, int write_fd)
{
	struct timeval	to;
	pid_t						pid;
	fd_set					read_fds;
	int							max, rc;
	CONTEXT					*ctx;
	char						log_msg[255];

	pid = getpid();
#ifdef SLAVE_DEBUG
	sprintf(log_msg,
					"0:0:LOG:Slave pid %d starting",
				 	pid);
	send_message(write_fd, log_msg);
#endif

	context_init();

	while(1)
		{
		FD_ZERO(&read_fds);
		to.tv_sec = 1;
		to.tv_usec = 0;

		FD_SET(read_fd, &read_fds);
		max = read_fd;

		ctx = context_head;
		while(ctx)
			{
			FD_SET(ctx->slave_ipc_read, &read_fds);
			if (max < ctx->slave_ipc_read)
				max = ctx->slave_ipc_read;
			ctx = ctx -> next;
			}
		if ((rc = select((max+1), &read_fds, NULL, NULL, &to)) < 0)
			{
			printf("Slave %d select failed\n",pid);
			continue;
			}
		else if (!rc)
			{
			continue;
			}
		if (FD_ISSET(read_fd, &read_fds))
			{
			process_msg_from_master(read_fd, write_fd);
			continue;
			}
		ctx = context_head;
		while(ctx)
			{
			if (FD_ISSET(ctx->slave_ipc_read, &read_fds))
				process_msg_from_thread(ctx, read_fd, write_fd);
			ctx = ctx -> next;
			}
		}
}

static void process_msg_from_master(int r_fd, int w_fd)
{
	char				*msg_buffer, *cmd, *node_id, *variable, *value, *new_name;
	int					rc;
	char				*saved_buffer;
	__uint32_t	len;

	if ((rc = read_message(r_fd, &msg_buffer, &len)) < 0)
		{
		printf("Slave unable to read msg from thread (%s)\n",strerror((-rc)));
		return;
		}
	saved_buffer = malloc(len);
	strcpy(saved_buffer, msg_buffer);

//	printf("Slave got msg (%s) from master\n",msg_buffer);
	if (!(cmd = strtok(msg_buffer, ":")))
		goto out;
	if (!(node_id = strtok(NULL, ":")))
		goto out;
	if (!strcasecmp(cmd, "NODE_NEW"))
		{
		if (!context_create(node_id))
			{
#error error msg not adjusted for module
			printf("SKEL Slave unable to create node context\n");
			goto out;
			}
		}
	else if (!strcasecmp(cmd, "NODE_VAR"))
		{
		if (!(variable = strtok(NULL,":")))
			goto out;

		/* BEGIN San mod 07/12/00 */
		value = &msg_buffer[((variable - msg_buffer)+(strlen(variable)+1))];
		/* END San mod 07/12/00 */

		if ((rc = context_variable_set(node_id, variable, value)) < 0)
			{
#error error msg not adjusted for module
			printf("SKEL Slave unable to set context variable %s (%s)\n",
						 variable,
						 strerror(-rc));
			goto out;
			}
		}
	else if (!strcasecmp(cmd, "CLIENT_DIS"))
		{
		char	*client_fd;
		int		fd;

		if (!(client_fd = strtok(NULL,":")))
			goto out;
		fd = atoi(client_fd);
		context_notify_disconnection_all(fd);
		goto out;
		}
	else if (!strcasecmp(cmd, "NODE_RENAME"))
		{
		if (!(new_name = strtok(NULL,":")))
			goto out;
		if ((rc = context_rename(node_id, new_name)) < 0)
			{
#error error msg not adjusted for module
			printf("SKEL Slave unable to rename context %s (%s)\n",
						 node_id,
						 strerror(-rc));
			goto out;
			}
		}
	else if (!strcasecmp(cmd, "NODE_DEL"))
		{
		if ((rc = context_destroy(node_id)) < 0)
			{
#error error msg not adjusted for module
			printf("SKEL Slave unable to destroy node context (%s)\n",strerror(-rc));
			goto out;
			}
		}
	else if (!strcasecmp(cmd, "NODE_IPC"))
		{
		if ((rc = context_send_ipc(node_id, saved_buffer)) < 0)
			{
#error error msg not adjusted for module
			printf("SKEL Slave unable to send IPC (%s)\n",strerror(-rc));
			goto out;
			}
		}
	else if (!strcasecmp(cmd,"SLAVE_SHUTDOWN"))
		{
		context_destroy_all();
		exit(0);
		}
	else
		{
#error error msg not adjusted for module
		printf("SKEL Slave received unknown msg from master (%s)\n",cmd);
		}
out:
	free(msg_buffer);
	free(saved_buffer);
}

static void process_msg_from_thread(CONTEXT *ctx, int read_fd, int write_fd)
{
	char				*msg_buffer;
	int					rc;
	__uint32_t	len;

	if ((rc = read_message(ctx->slave_ipc_read, &msg_buffer, &len)) < 0)
		{
		printf("Slave unable to read msg from thread (%s)\n",strerror((-rc)));
		return;
		}
	// printf("Slave got msg (%s) from thread\n",msg_buffer);
	/* Forward message to master */
	send_message(write_fd, msg_buffer);
	free(msg_buffer);
}

static int read_message(int fd, char **buffer, __uint32_t *len)
{
	__uint32_t	net_len;
	int					rc;
	__uint32_t	brtr;
	char				*offset;

	brtr = sizeof(net_len);
	offset = (char *) &net_len;
	while(brtr)
		{
		if ((rc = read(fd, (void *) offset, brtr)) < 0)
			return(rc);
		else if (!rc)
			return(-1);
		brtr -= rc;
		offset += rc;
		}
	*len = ntohl(net_len);

	if (!(*buffer = malloc(*len)))
		{
		printf("Unable to allocate memory of size %d (0x%x)\n",*len,*len);
		return(-ENOMEM);
		}
	memset(*buffer, 0, *len);
	brtr = *len;
	offset = *buffer;
	while(brtr)
		{
		if ((rc = read(fd, (void *) offset, brtr)) < 0)
			return(rc);
		else if (!rc)
			return(-1);
		brtr -= rc;
		offset +=rc;
		}
	return(0);
}

static int send_message(int fd, char *msg)
{
  __uint32_t  len;
  __uint32_t  net_len;
  int         rc;

  len = strlen(msg)+1;
  net_len = htonl(len);
  if ((rc = write (fd, (void *) &net_len, sizeof(net_len))) < 0)
    return(rc);
  if ((rc = write (fd, (void *) msg, len)) < 0)
    return(rc);
  return(0);
}

