#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <sys/time.h>
#include <libloose.h>

static void config_cb(__uint32_t job_id, int client_fd, char *msg);
static void deinit_cb(__uint32_t job_id, int client_fd, char *msg);
static void discon_cb(__uint32_t job_id, int client_fd, char *msg);
static void ipc_cb(char *node_id, __uint32_t job_id, int client_fd, char *msg);
static int timer_cb(void *arg);

#define TRUE	1
#define FALSE	!TRUE
#define KNOWN_UNUSED_PARAM(x) { int __bleh; __bleh = (int) x;}

typedef struct	nodelist
	{
	char						node_id[64];
	unsigned int		ip_address;

	struct nodelist	*next;
	struct nodelist	*prev;
	} NODELIST;

static NODELIST	*node_create(char *node_id);
static NODELIST	*lookup_node(char *node_id);
static void	node_destroy(NODELIST	*node);

static NODELIST	*nodelist_head = NULL;
static NODELIST	*nodelist_tail = NULL;

int main()
{
	lm_init();
	lm_nexxus_connect();
	if (lm_register("Ping Tester",
									"ICMP Echo Tester",
									"Sends ICMP ECHO messages to a remote host to test connectivity",
									"San Mehat (nettwerk@valinux.com)",
									"ICMP_ECHO",
									1,
									0) < 0)
		{
		printf("Unable to register module with Nexxus\n");
		exit(-1);
		}
	lm_main_loop(config_cb, deinit_cb, discon_cb, ipc_cb);
exit(0);
}

static int timer_cb(void *arg)
{
  KNOWN_UNUSED_PARAM(arg);

	printf("timer_cb(): Test timer fired!\n");
  return(TRUE);
}

static void config_cb(__uint32_t job_id, int client_fd, char *msg)
{
	NODELIST	*node;
	char			*cmd_module;
	char			*node_id;

  KNOWN_UNUSED_PARAM(job_id);
  KNOWN_UNUSED_PARAM(client_fd);
	cmd_module = strtok(msg, ":");
	node_id = strtok(NULL, ":");

	node = lookup_node(node_id);

	if (!strncasecmp(cmd_module, "NODE", 4))
		node = node_create(node_id);
	else if (!strncasecmp(cmd_module, "END_NODE", 8))
		{
		struct	in_addr	addr;

		addr.s_addr = node->ip_address;
		printf("[Ping Tester] Node '%s' (ip %s) ready\n",
					node_id,inet_ntoa(addr));
		}
	else if (!strncasecmp(cmd_module, "ADDITION", 8))
		{
		node = node_create(node_id);
		}
	else if (!strncasecmp(cmd_module, "DELETION", 8))
		{
		node_destroy(node);
		}
	else if (!strncasecmp(cmd_module, "RENAME", 6))
		{
		char	*new_name;
		
		if (!(new_name = strtok(NULL,":")))
			{
			printf("[Ping Tester] Malformatted cfg msg\n");
			return;
			}
		strcpy(node->node_id, new_name);
		}
	else if (!strncasecmp(cmd_module, "GLOBAL", 6))
		{
		char	*variable;
		char	*value;
		
		if (!(variable = strtok(NULL,":")))
			{
			printf("[Ping Tester] Malformatted cfg msg\n");
			return;
			}
		if (!(value = strtok(NULL,":")))
			{
			printf("[Ping Tester] Malformatted cfg msg\n");
			return;
			}
		if (!strcmp(variable, "IP_ADDRESS"))
			node->ip_address = inet_addr(value);
		}
	else if (!strncasecmp(cmd_module, "ICMP_ECHO", 9))
		{
		}
	else
		printf("[Ping Tester] Received unknown config event <%s>\n",cmd_module);
}

static void deinit_cb(__uint32_t job_id, int client_fd, char *msg)
{
  KNOWN_UNUSED_PARAM(job_id);
  KNOWN_UNUSED_PARAM(client_fd);
  KNOWN_UNUSED_PARAM(msg);

	printf("deinit_cb(): Received request to shutdown\n");
	lm_nexxus_disconnect();
	exit(0);
}

static void discon_cb(__uint32_t job_id, int client_fd, char *msg)
{
  KNOWN_UNUSED_PARAM(job_id);
  KNOWN_UNUSED_PARAM(msg);

	printf("discon_cb(): Received disconnection notification for client fd %d\n",
				 client_fd);
}

static void ipc_cb(char *node_id, __uint32_t job_id, int client_fd, char *msg)
{
  KNOWN_UNUSED_PARAM(node_id);
  KNOWN_UNUSED_PARAM(job_id);
  KNOWN_UNUSED_PARAM(client_fd);

	printf("ipc_cb(): Received <%s>\n",msg);
}

static NODELIST	*node_create(char *node_id)
{
	NODELIST	*new;

	if (!(new = (NODELIST	*) malloc(sizeof(NODELIST))))
		return(0);
	memset(new, 0, sizeof(NODELIST));
	if (!nodelist_head)
		nodelist_head = nodelist_tail = new;
	else
		{
		nodelist_tail->next = new;
		new->prev = nodelist_tail;
		nodelist_tail = new;
		}
	strcpy(new->node_id, node_id);
	return(new);
}

static NODELIST	*lookup_node(char *node_id)
{
	NODELIST	*scan;

	if (!node_id)
		return(NULL);
	scan = nodelist_head;
	while(scan)
		{
		if (!strcmp(node_id, scan->node_id))
			return(scan);
		scan = scan->next;
		}
	return(NULL);
}

static void	node_destroy(NODELIST	*node)
{
  if (node == nodelist_head)
    {
    nodelist_head = nodelist_head->next;
    if (nodelist_head)
      nodelist_head->prev = NULL;
    else
      nodelist_tail = NULL;
    } 
  else if (node == nodelist_tail)
    {
    nodelist_tail = nodelist_tail->prev;
    if (nodelist_tail)
      nodelist_tail->next = NULL;
    else
      nodelist_head = NULL;
    } 
  else
    {
    node->next->prev = node->prev;
    node->prev->next = node->next;
    }
  free(node);
}
